package editorListener.handlers;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

/**
 * This class provides functionality to allow different file types opened by the
 * editor to be listened to.
 * 
 * In doing so, this class provides some abstraction towards a common set of
 * Dispatch clients across all file types. However, most dispatches rely on the
 * JavaEditor's SourceViewer which is not accessible to superclasses. This may
 * seriously limit extensibility.
 * 
 * @author Matthew Harward
 * @version 4/8/2009
 */
public class AcceptableFileTypeHandler {

	private Set<String> allowedFileTypes;
	private List<String> allowedClassTypes;

	/**
	 * Default constructor, provides default Java implementation.
	 */
	public AcceptableFileTypeHandler() {
		allowedFileTypes = new HashSet<String>();
		allowedFileTypes.add("Java Source File");
		allowedClassTypes = new ArrayList<String>();
		allowedClassTypes.add("JavaEditor");
	}

	/**
	 * Constructor for any type of file.
	 */
	public AcceptableFileTypeHandler(Set<String> allowedFileTypes,
			List<String> allowedClassTypes) {
		this.allowedFileTypes = allowedFileTypes;
		this.allowedClassTypes = allowedClassTypes;
	}

	/**
	 * Checks the provided object against the existing set of acceptable class
	 * names.
	 * 
	 * @param obj
	 *            the <code>Object</code> to be checked
	 * @return true if the object is acceptable and can be found, else false.
	 */
	public boolean allowableObject(Object obj) {
		boolean allowed = false;
		for (String className : allowedClassTypes) {
			try {
				if (Class.forName(className).isInstance(obj))
					allowed = true;
			} catch (ClassNotFoundException e) {
			}
		}
		return allowed;
	}

	/**
	 * Checks the underlying file type names against an existing set.
	 * 
	 * @param name
	 *            the file name to be checked
	 * @return true if the file is acceptable, else false.
	 */
	public boolean checkFileType(String name) {
		boolean acceptable = false;
		for (String fileType : allowedFileTypes) {
			if (fileType.equals(name))
				acceptable = true;
		}
		return acceptable;
	}
}
