package editorListener.handlers;

import java.util.Observable;

import org.eclipse.jdt.internal.ui.javaeditor.JavaEditor;
import org.eclipse.jface.text.IDocument;
import org.eclipse.ui.IEditorReference;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PlatformUI;

/**
 * This singleton class is part of an extensible observer pattern providing a
 * concrete class for all listening capabilities of this plug-in.
 * 
 * Possible functionality to add: - Reporting of all open windows. Note: Eclipse
 * will not allow the same resource to be opened by the same editor in the same
 * workbench multiple times. This does not apply across perspectives or windows
 * in the same instance.
 * 
 * @author Matthew Harward
 * @version 3/8/2009
 */
@SuppressWarnings("restriction")
public class EditorListener extends Observable {

	// Fields
	protected UpdatedEditor curEditor;
	protected AcceptableFileTypeHandler typeHandler;

	// The singleton
	protected static EditorListener eListener;

	/**
	 * Default constructor. Subclasses should call.
	 */
	protected EditorListener() {
		typeHandler = new AcceptableFileTypeHandler();
	}

	/**
	 * Singleton instantiation.
	 * 
	 * @return the instance of this class.
	 */
	public static EditorListener getInstance() {
		if (eListener == null)
			eListener = new EditorListener();
		return eListener;
	}

	/**
	 * This method attempts to determine if this is a new editor to the current
	 * editor held by this object.
	 * 
	 * @param doc
	 *            the document to check.
	 * @return true is the editor is new, else false.
	 */
	private boolean isEditorNew(IDocument doc) {
		if (curEditor != null && curEditor.getDocument() != null && doc != null) {
			if (!doc.equals(curEditor.getDocument())) {
				return true;
			}
		}
		return false;
	}

	/**
	 * Method to be called by dispatches when a document is opened. Provides an
	 * observer notification.
	 * 
	 * @param doc
	 *            the document
	 */
	public void documentOpen(IDocument doc) {
		if (curEditor == null || isEditorNew(doc))
			callUpdate(doc, EditorState.DocumentOpened);
		else
			updateCurrentDocument(doc);
	}

	/**
	 * Method to be called by dispatches when a document is closed.
	 * 
	 * @param doc
	 *            the document
	 */
	public void documentClosed(IDocument doc) {
		curEditor = null;
	}

	/**
	 * Method to be called by dispatches when a document is brought into focus.
	 * Provides an observer notification.
	 * 
	 * @param doc
	 *            the document
	 */
	public void changeFocus(IDocument doc) {
		if (curEditor != null)
			callUpdate(doc, EditorState.DocumentInFocus);
		else
			documentOpen(doc);
	}

	/**
	 * Method to be called by dispatches when a document is modified. Provides
	 * an observer notification.
	 * 
	 * @param doc
	 *            the document
	 */
	public void updateCurrentDocument(IDocument doc) {
		if (curEditor != null)
			callUpdate(doc, EditorState.DocumentModified);
		else
			documentOpen(doc);
	}

	/**
	 * This method calls the updates and notifies the observer.
	 */
	private void callUpdate(IDocument doc, EditorState state) {
		IWorkbenchPart iwp = getWorkbenchPart(doc, false);
		curEditor = new UpdatedEditor(doc, iwp, state);
		// Check services
		if (!ServiceChecker.getInstance().servicesRunning())
			ServiceChecker.getInstance().restartAllServices();
		// Provide observer notification
		setChanged();
		notifyObservers(curEditor);
	}

	/**
	 * A static method to get the workbench part from a document. Attempts a
	 * dynamic method first.
	 * 
	 * @param doc
	 *            the document
	 * @return the workbench part corresponding to the document
	 */
	public static IWorkbenchPart getWorkbenchPart(IDocument doc, boolean restore) {
		IWorkbenchPart finalPart = null;
		try{
		if ((finalPart = getActiveJavaEditor()) == null) {
			IWorkbenchWindow[] windows = PlatformUI.getWorkbench()
					.getWorkbenchWindows();
			for (IWorkbenchWindow window : windows) {
				for (IWorkbenchPage page : window.getPages()) {
					for (IEditorReference ref : page.getEditorReferences()) {
						IWorkbenchPart part = (IWorkbenchPart) ref
								.getEditor(restore);
						if (part instanceof JavaEditor) {
							IDocument foundDoc = ((JavaEditor) part)
									.getViewer().getDocument();
							if (doc.toString().equals(foundDoc.toString()))
								finalPart = part;
						}
					}
				}
			}
		}
		} catch (Exception e) {
		}
		return finalPart;
	}

	/**
	 * Attempts to get the currently the active JavaEditor.
	 * 
	 * @return the workbench part of the editor, if there is one open and
	 *         visible, else <code>null</code>.
	 */
	public static IWorkbenchPart getActiveJavaEditor() {
		IWorkbenchPart part = (IWorkbenchPart) PlatformUI.getWorkbench()
				.getActiveWorkbenchWindow().getActivePage().getActiveEditor();
		if (part instanceof JavaEditor)
			return part;
		else
			return null;
	}

	/**
	 * @return the current file type handler.
	 */
	public AcceptableFileTypeHandler getTypeHandler() {
		return typeHandler;
	}

	/**
	 * @param typeHandler
	 *            set the file type handler. This can be used to provide
	 *            non-Java listening abilities.
	 */
	public void setTypeHandler(AcceptableFileTypeHandler typeHandler) {
		this.typeHandler = typeHandler;
	}
}
