package metricsOverlay;

import java.util.HashSet;
import java.util.Set;

import metricsOverlay.model.augmentation.Augmentation;
import metricsOverlay.model.augmentation.BackgroundLineAugmentation;
import metricsOverlay.model.augmentation.BackgroundSectionAugmentation;
import metricsOverlay.model.augmentation.MarginAugmentation;
import metricsOverlay.model.augmentation.UnderlineAugmentation;
import metricsOverlay.model.mappings.AugmentExponentialIStrategy;
import metricsOverlay.model.mappings.AugmentLogIStrategy;
import metricsOverlay.model.mappings.InterpolationStrategy;
import metricsOverlay.model.mappings.LinearIStrategy;
import metricsOverlay.model.mappings.MetricExponentialIStrategy;
import metricsOverlay.model.mappings.MetricLogIStrategy;
import metricsOverlay.model.metrics.CompositeIntegerRange;
import metricsOverlay.model.metrics.CompositeRealRange;
import metricsOverlay.model.metrics.IntegerRange;
import metricsOverlay.model.metrics.OrderedNominalRange;
import metricsOverlay.model.metrics.Range;
import metricsOverlay.model.metrics.RealRange;

public class OverlayRegistry {

	private Set<Class<? extends Augmentation>> augmentations = 
		new HashSet<Class<? extends Augmentation>>();
	
	private Set<Class<? extends Range>> ranges = 
		new HashSet<Class<? extends Range>>();
	
	private Set<Class<? extends InterpolationStrategy>> interpolations = 
		new HashSet<Class<? extends InterpolationStrategy>>();
	
	private static OverlayRegistry oReg;
	
	private OverlayRegistry() {
		//Add Augmentations
		addAugmentation(MarginAugmentation.class);
		addAugmentation(BackgroundLineAugmentation.class);
		addAugmentation(BackgroundSectionAugmentation.class);
		addAugmentation(UnderlineAugmentation.class);
		//Add Ranges
		addRange(RealRange.class);
		addRange(IntegerRange.class);
		addRange(OrderedNominalRange.class);
		addRange(CompositeRealRange.class);
		addRange(CompositeIntegerRange.class);
		//Add Interpolations
		addInterpolation(LinearIStrategy.class);
		addInterpolation(MetricExponentialIStrategy.class);
		addInterpolation(AugmentExponentialIStrategy.class);
		addInterpolation(MetricLogIStrategy.class);
		addInterpolation(AugmentLogIStrategy.class);
	}
	
	public static OverlayRegistry getInstance(){
		if(oReg == null)
			oReg = new OverlayRegistry();
		return oReg;
	}

	public Augmentation getAugmentation(String string) {
		Augmentation n = null;
		for(Class<? extends Augmentation> aug : augmentations){
			try {
				n = aug.newInstance();
				if(n.getType().equals(string))
					return n;
			} catch (IllegalArgumentException e) {} 
			catch (SecurityException e) {} 
			catch (IllegalAccessException e) {} 
			catch (InstantiationException e) {}
		}
		return null;
	}
	
	public InterpolationStrategy getInterpolation(String string) {
		InterpolationStrategy n = null;
		for(Class<? extends InterpolationStrategy> inter : interpolations){
			try {
				n = inter.newInstance();
				if(n.getType().equals(string))
					return n;
			} catch (IllegalArgumentException e) {} 
			catch (SecurityException e) {} 
			catch (IllegalAccessException e) {} 
			catch (InstantiationException e) {}
		}
		return null;
	}

	public void addAugmentation(Class<? extends Augmentation> augClass) {
		augmentations.add(augClass);
	}
	
	public void addRange(Class<? extends Range> rangeClass) {
		ranges.add(rangeClass);
	}
	
	public void addInterpolation(Class<? extends InterpolationStrategy> interClass) {
		interpolations.add(interClass);
	}
	
	public String[] getAllAugNames(){
		String[] stArr = new String[augmentations.size()];//[2];
		int count = 0;
		for(Class<? extends Augmentation> cl : augmentations){
			try {
				final String[] row = new String[2];
				row[0] = (String) cl.newInstance().getType();
				row[1] = row[0];
				stArr[count++] = row[0];
			} catch (IllegalArgumentException e) {
			} catch (SecurityException e) {
			} catch (IllegalAccessException e) {
			} catch (InstantiationException e) {}
		}
		return stArr;
	}
	
	public String[] getAllRangeNames(){
		String[] stArr = new String[ranges.size()];
		int count = 0;
		for(Class<? extends Range> cl : ranges){
			try {
				final String[] row = new String[2];
				row[0] = (String) cl.newInstance().getType();
				row[1] = row[0];
				stArr[count++] = row[0];
			} catch (IllegalArgumentException e) {
			} catch (SecurityException e) {
			} catch (IllegalAccessException e) {
			} catch (InstantiationException e) {}
		}
		return stArr;
	}
	
	public String[] getAllInterNames(){
		String[] stArr = new String[interpolations.size()];
		int count = 0;
		for(Class<? extends InterpolationStrategy> cl : interpolations){
			try {
				final String[] row = new String[2];
				row[0] = (String) cl.newInstance().getType();
				row[1] = row[0];
				stArr[count++] = row[0];
			} catch (IllegalArgumentException e) {
			} catch (SecurityException e) {
			} catch (IllegalAccessException e) {
			} catch (InstantiationException e) {}
		}
		return stArr;
	}
	
}
