package metricsOverlay.augmentationStrategies;

import java.lang.reflect.Method;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import metricsOverlay.handlers.MetricFile;
import metricsOverlay.model.MetricSection;
import metricsOverlay.model.mappings.Mapping;

import org.eclipse.jdt.internal.ui.javaeditor.CompilationUnitEditor;
import org.eclipse.jface.text.source.projection.ProjectionViewer;
import org.eclipse.swt.custom.StyledText;
import org.eclipse.swt.graphics.Color;
import org.eclipse.ui.texteditor.AbstractTextEditor;

/**
 * This abstract class provides the required methods for a display strategy.
 * 
 * @author Matthew Harward
 * @version 26/8/2009
 */
@SuppressWarnings("restriction")
public abstract class AugmentationStrategy {

	// Declared fields
	/**
	 * This field determines whether the display strategy is active. It should
	 * be set using <code>setActive</code> and the value retrieved using
	 * <code>isActive</code>.
	 */
	protected boolean active;
	/**
	 * The current <code>MetricFile</code> in use.
	 */
	protected MetricFile curMFile;
	/**
	 * The <code>JavaEditor</code> in use.
	 */
	protected CompilationUnitEditor editor;
	/**
	 * The viewer used to display the current <code>JavaEditor</code>.
	 */
	protected ProjectionViewer viewer;
	/**
	 * The widget used to represent the viewer of the <code>JavaEditor</code>.
	 */
	protected StyledText textWidget;
	/**
	 * The name of the current mapping
	 */
	protected Mapping mapping;

	/**
	 * This class updates the display strategy, it should be overriden by subclasses.
	 * super must be called at the start if the method
	 * 
	 * @param mf
	 *            the metric file describing the current file and the metrics to
	 *            be applied.
	 * @param mm
	 *            the name of the current mapping this display strategy belongs
	 *            to.
	 */
	public void update(MetricFile mf, Mapping mm) {
		this.curMFile = mf;
		this.mapping = mm;
		this.editor = (CompilationUnitEditor) curMFile.getWorkbenchPart();
		try {
			// Use reflection to get the viewer for the editor.
			Method getSourceViewer = AbstractTextEditor.class
					.getDeclaredMethod("getSourceViewer");
			getSourceViewer.setAccessible(true);
			final ProjectionViewer viewer = (ProjectionViewer) getSourceViewer
					.invoke(editor);
			this.viewer = viewer;
			// Get the widget
			textWidget = viewer.getTextWidget();
			setActive(true);
		} catch (Exception e) {
			System.err.println("Unable to intialise display strategy");
			e.printStackTrace();
		}
	}

	/**
	 * Sets this display strategy as active.
	 * 
	 * @param active
	 *            if true, the display strategy is set to active, else inactive.
	 */
	public void setActive(boolean active) {
		this.active = active;
	}

	/**
	 * Returns whether the display strategy is active or not.
	 * 
	 * @return true if active, else false.
	 */
	public boolean isActive() {
		return active;
	}

	/**
	 * This method takes in a string and parses it into an SWT compatible
	 * colour. The input string should take the form of {255, 255, 255}. This
	 * method is flexible enough to take any input string, as long as it
	 * contains 3 separate numbers of between 0-255. If the number provided is
	 * not separated, each section will be greedily assumed to be 3 digits in
	 * length, i.e. "245211001232349" will produce
	 * <code>Color {245, 211, 1}</code>. This means that this method is able to
	 * parse the <code>toString</code> method of
	 * <code>org.eclipse.swt.graphics.Color</code>
	 * 
	 * @param val
	 *            the entered string
	 * @return the parsed colour if successful, or <code>null</code>
	 */
	public static Color parseColor(String val) {
		Color finalC = null;
		try {
			String value = val.trim();
			Pattern a = Pattern.compile("[0-9]{1,3}");
			Matcher m = a.matcher(value);
			m.find();
			int part1 = Integer.parseInt(m.group());
			m.find();
			int part2 = Integer.parseInt(m.group());
			m.find();
			int part3 = Integer.parseInt(m.group());
			if (part1 > 255 || part2 > 255 || part3 > 255)
				throw new Exception();
			finalC = new Color(null, part1, part2, part3);
		} catch (Exception e) {
		}
		return finalC;
	}
	
	/**
	 * Gets the offset, in terms of a number of characters
	 * from the start of the document to the start of the provided metric
	 * section.
	 * 
	 * @param mSec the metric section to be examined
	 * @return the number of characters
	 */
	protected int getOffset(MetricSection mSec) {
		int startLine = mSec.getStartLine();
		int startCol = mSec.getStartCol();
		return startCol + textWidget.getOffsetAtLine(startLine);
	}
	
	/**
	 * Gets the length of a metric section in terms of the number
	 * of characters it occupies in the current document.
	 * 
	 * @param mSec the metric section to be examined
	 * @return the number of characters
	 */
	protected int getLength(MetricSection mSec) {
		int endLine = mSec.getEndLine();
		int endCol = mSec.getEndCol();
		return textWidget.getOffsetAtLine(endLine) + endCol - getOffset(mSec);
	}

	/**
	 * Subclasses should override this method
	 */
	public abstract void dispose();
}
