package metricsOverlay.handlers;

import metricsOverlay.Activator;
import metricsOverlay.DefaultAugmentations;
import metricsOverlay.augmentationStrategies.ForcedDisplayStrategy;
import metricsOverlay.model.augmentation.Augmentation;
import metricsOverlay.model.augmentation.other.AugmentationManager;
import metricsOverlay.model.mappings.Mapping;
import metricsOverlay.preferences.MetricsOverlayPreferencePage;

import org.eclipse.jface.preference.IPreferenceStore;

/**
 * This class keeps track of system components and makes sure that views and
 * other elements are updated as necessary.
 * 
 * @author Matthew Harward
 * @version 2/9/2009
 */
public class MetricFileHandler {

	private static MetricFileHandler fHandler;
	private DocumentTracker dTracker;
	private AugmentationManager disManager;
	private UpdateViewerAction updateViewer;
	private MetricFile curMf;

	private boolean allOn;

	private MetricFileHandler() {
		dTracker = DocumentTracker.getInstance();
		disManager = new AugmentationManager();
		updateViewer = new UpdateViewerAction();
		allOn = false;
	}

	public static MetricFileHandler getInstance() {
		if (fHandler == null) {
			fHandler = new MetricFileHandler();
		}
		return fHandler;
	}

	public synchronized void update(MetricFile mf, boolean internal) {
		if (mf != null) {
			if (!allOn) {
				mf.getMetricDocument().disableAllMappings();
			}
			if (curMf == null) {
				// Add default augmentations
				for (Augmentation au : DefaultAugmentations
						.getDefaultAugmentations()) {
					mf.getMetricDocument().addAugmentation(au);
				}
			} else if (internal) {
				// If it is being accessed internally this will not be an issue.
				curMf = mf;
				// ConcurrentModificationExceptions will occur if mf and curMf
				// are the
				// same object and they are external..
			} else if (updatedFileOfCurrent(mf)) {
				System.out.println("This is an updated file");
				// if the number and names of actual metrics have not changed
				if (metricsUnchanged(mf)) {
					mf = new ModelRationaliser().rationalise(curMf, mf);
				}
				// if metric names have been removed, remove invalid metrics,
				// disable all affected mappings and remove their metrics
				// If the file has changed...
				// We want to keep settings for each file
			} else if (checkPreserveSettings()) {

				// We want new settings for each file
			} else {
				// Do we currently have an (older) instance of this metric file?
				if (dTracker.contains(mf)) {
					// If yes, rationalise with current view settings, update
					// view if necessary
				}

			}

			// TEMP>>>
			// Add default augmentations
			for (Augmentation au : DefaultAugmentations
					.getDefaultAugmentations()) {
				mf.getMetricDocument().addAugmentation(au);
			}

			

			// Update file here and in document tracker
			curMf = mf;
			dTracker.addMetricFile(mf);
			// Call AugmentationManager to display file
			doUpdate();
		}
	}
	
	public void updateMapping(Mapping map){
		Mapping mm = curMf.getMetricDocument().getMapping(map.getName());
		if(mm != null && mm.equals(map)){
			//Just update
		} else {
			if(mm != null)
				curMf.getMetricDocument().removeMapping(map.getName());
			curMf.getMetricDocument().addMapping(map);
		}
		doUpdate();
	}

	private void doUpdate() {
		if(curMf != null) {
		// Update table view...
		MetricEvent m = new MetricEvent(curMf);
		updateViewer.runWithEvent(m);
		if (allOn)
			disManager.updateDisplay(curMf);
		forceUpdate();
		}
	}

	private boolean metricsUnchanged(MetricFile mf) {
		return true;
	}

	private boolean updatedFileOfCurrent(MetricFile mf) {
		if (mf.getPath().toString().equals(curMf.getPath().toString())) {
			return true;
		}
		return false;
	}

	private boolean checkPreserveSettings() {
		IPreferenceStore store = Activator.getDefault().getPreferenceStore();
		return store.getBoolean(MetricsOverlayPreferencePage.PRESERVE_SETTINGS);
	}

	public void toggleAllOn() {
		allOn = true;
		doUpdate();
		
	}

	public void toggleAllOff() {
		allOn = false;
		disManager.clearAllDisplays();
		
	}

	public void forceUpdate() {
		ForcedDisplayStrategy fds = new ForcedDisplayStrategy();
		fds.update(curMf, null);
	}

	public UpdateViewerAction getViewerUpdateAction() {
		return updateViewer;
	}

	public MetricFile getCurrentMFile() {
		return curMf;
	}

}
