package metricsOverlay.model;

import java.sql.Timestamp;
import java.util.Collections;
import java.util.LinkedHashSet;
import java.util.Set;

import metricsOverlay.model.augmentation.Augmentation;
import metricsOverlay.model.mappings.Mapping;
import metricsOverlay.model.mappings.MappingState;
import metricsOverlay.model.metrics.ActualMetric;
import metricsOverlay.model.metrics.Metric;

import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.Path;

/**
 * A metric document is a set of metric sets and there mappings, with a path
 * that indicates the document they relate to. A metric document also includes a
 * time stamp so its validity can be confirmed.
 * 
 * @author Matthew Harward
 * @version 14/7/09
 */
public class MetricDocument implements Comparable<MetricDocument> {

	// Data Fields
	private Set<Metric> metrics;
	private Set<Mapping> mappings;
	private Set<Augmentation> augmentations;
	
	// Usability Fields
	private IPath fileName;
	private Timestamp timestamp;
	private boolean disableAll;
	private static long AGECOMPARE = 5000;

	/**
	 * Main constructor.
	 * 
	 * @param fileName
	 *            initialise the document with a specific path.
	 */
	public MetricDocument(IPath fileName) {
		mappings = Collections.synchronizedSet(new LinkedHashSet<Mapping>());
		metrics = Collections.synchronizedSet(new LinkedHashSet<Metric>());
		augmentations = Collections.synchronizedSet(new LinkedHashSet<Augmentation>());
		this.fileName = fileName;
		updateTimeStamp();
		disableAll = false;
	}

	/**
	 * Alternate constructor
	 * 
	 * @param fileName
	 *            initialise the document with a string based path. This path
	 *            needs to be in a standard path format.
	 */
	public MetricDocument(String fileName) {
		this(Path.fromOSString(fileName));
	}

	/**
	 * Update the time stamp on this document to the current time.
	 */
	public void updateTimeStamp() {
		Timestamp ts = new Timestamp(System.currentTimeMillis());
		this.timestamp = ts;
	}

	/**
	 * Sets the time stamp on this document to a specific time.
	 * 
	 * @param ts
	 *            the time stamp to be set
	 */
	public void setTimeStamp(Timestamp ts) {
		this.timestamp = ts;
	}

	/**
	 * Sets the time stamp on this document to a particular system time
	 * 
	 * @param timeInMillis
	 *            standard time in millis
	 */
	public void setTimeStamp(long timeInMillis) {
		Timestamp ts = new Timestamp(timeInMillis);
		this.timestamp = ts;
	}

	/**
	 * @return the current time stamp
	 */
	public Timestamp getTimeStamp() {
		return timestamp;
	}

	/**
	 * @return an IPath representing the documents current associated file.
	 */
	public IPath getFileName() {
		return fileName;
	}

	/**
	 * Sets the file name to a specific path
	 * 
	 * @param fileName
	 *            the new path
	 */
	public void setFileName(IPath fileName) {
		this.fileName = fileName;
	}

	/**
	 * Sets the new file name from a given string.
	 * 
	 * @param fileName
	 *            the new String in a path format
	 */
	public void setFileNameFromString(String fileName) {
		this.fileName = Path.fromOSString(fileName);
	}

	/**
	 * Counts the number of mappings that are not stubs.
	 * 
	 * @return the number of mappings (not including stubs)
	 */
	public synchronized int numMappings() {
		int count = 0;
		for (Mapping m : mappings) {
			if (!m.getMappingState().equals(MappingState.StubMapping))
				count++;
		}
		return count;
	}
	
	public synchronized int numMetrics() {
		return metrics.size();
	}
	
	public synchronized int numAugmentations() {
		return augmentations.size();
	}
	
	/**
	 * @return returns the number of declared mappings (not including stubs)
	 */
	public synchronized int numDeclaredMappings() {
		int count = 0;
		for (Mapping m : mappings) {
			if (m.getMappingState().ordinal() > 1)
				count++;
		}
		return count;
	}
	
	/**
	 * @return returns the number of active mappings.
	 */
	public synchronized int numActiveMappings() {
		int count = 0;
		for (Mapping m : mappings) {
			if (m.getMappingState().ordinal() > 2)
				count++;
		}
		return count;
	}

	/**
	 * @return an immutable set of mapped metrics.
	 */
	public synchronized Set<Mapping> getAllMappings() {
		if(disableAll) disableAllMappings();
		//return Collections.unmodifiableSet(mappings);
		return mappings;
	}
	
	public synchronized Set<Metric> getAllMetrics() {
		//return Collections.unmodifiableSet(metrics);
		return metrics;
	}
	
	public synchronized Set<Augmentation> getAllAugmentations() {
		//return Collections.unmodifiableSet(augmentations);
		return augmentations;
	}

	/**
	 * Overriden toString method.
	 */
	public String toString() {
		return "Metrics for file: " + fileName.toOSString() + "\n"
				+ mappings.toString();
	}

	public synchronized void disableAllMappings() {
		disableAll = true;
		for(Mapping mm : mappings){
			if(mm.getMappingState().ordinal() > 2)
				mm.setMappingState(MappingState.DeclaredMapping);
		}
	}
	
	public synchronized void enableAllMappings() {
		disableAll = false;
		for(Mapping mm : mappings){
			if(mm.getMappingState().equals(MappingState.DeclaredMapping))
				mm.setMappingState(MappingState.ActiveMapping);
		}
	}

	public synchronized Mapping getMapping(String mappingName) {
		for(Mapping m : mappings){
			if(m.getName().equals(mappingName))
				return m;
		}
		return null;
	}
	
	public synchronized Metric getMetric(String metricName) {
		for(Metric m : metrics){
			if(m.getName().equals(metricName))
				return m;
		}
		return null;
	}
	
	public synchronized Augmentation getAugmentation(String displayName) {
		for(Augmentation d : augmentations){
			if(d.getName().equals(displayName))
				return d;
		}
		return null;
	}
	
	public synchronized void addMapping(Mapping mapping) {
		if(mapping.getMetric() != null)
			metrics.add(mapping.getMetric());
		if(mapping.getAugmentation() != null)
			augmentations.add(mapping.getAugmentation());
		mappings.add(mapping);
	}
	
	public synchronized void addMetric(Metric met){
		metrics.add(met);
	}
	
	public synchronized void addAugmentation(Augmentation augmentation) {
		if(isAugmentationNameUnique(augmentation.getName()))
			augmentations.add(augmentation);
	}

	public synchronized boolean isMappingNameUnique(String name) {
		boolean unique = true;
		for(Mapping a : mappings){
			if(a.getName().equals(name)){
				unique = false;
				break;
			}
		}
		return unique;
	}

	public synchronized boolean isAugmentationNameUnique(String name){
		boolean unique = true;
		for(Augmentation a : augmentations){
			if(a.getName().equals(name)){
				unique = false;
				break;
			}
		}
		return unique;
	}
	
	public synchronized boolean isMetricNameUnique(String name) {
		boolean unique = true;
		for(Metric a : metrics){
			if(a.getName().equals(name)){
				unique = false;
				break;
			}
		}
		return unique;
	}

	@Override
	public synchronized int compareTo(MetricDocument o) {
		if (mappings.containsAll(o.mappings)){
			if(Math.abs(timestamp.getTime()-o.timestamp.getTime()) < AGECOMPARE){
				return 1;
			}
		}
		return -1;
	}

	public synchronized void setAllMappingState(MappingState mState) {
		for(Mapping map : mappings){
			map.setMappingState(mState);
		}
	}

	public synchronized void removeAugmentation(String name) {
		Augmentation found = null;
		for(Augmentation m : augmentations){
			if(m.getName().equals(name)){
				found = m;
				break;
			}
		}
		augmentations.remove(found);
		for(Mapping m : mappings){
			if(m.getAugmentation() != null && m.getAugmentation().getName().equals(name))
				m.clearAugmentation();
		}
	}

	public synchronized void removeMapping(String name) {
		Mapping found = null;
		for(Mapping m : mappings){
			if(m.getName().equals(name)){
				found = m;
				break;
			}
		}
		mappings.remove(found);
	}

	public synchronized void removeMetric(String name) {
		Metric found = null;
		for(Metric m : metrics){
			if(m.getName().equals(name)){
				found = m;
				break;
			}
		}
		if(!(found instanceof ActualMetric))
			metrics.remove(found);
	}
}
