package metricsOverlay.model.augmentation.other;

import java.util.HashSet;
import java.util.Set;


/**
 * This class provides a set of shapes that can be used with a the SWT GC of the
 * text widget used to display the java editor to render specific shapes.
 * 
 * @author Matthew Harward
 * @version 9/7/2009
 */
public final class ShapeManager {
	// General shapes
	private static final int[] star = { 6, 0, 7, 4, 12, 5, 8, 7, 10, 12, 6, 8,
			0, 11, 3, 7, 0, 4, 4, 4 };
	private static final int[] smallSquare = { 4, 4, 9, 4, 9, 9, 4, 9 };
	private static final int[] largeSquare = { 2, 2, 11, 2, 11, 11, 2, 11 };
	// Arrows
	private static final int[] leftArrow = { 7, 1, 1, 7, 7, 13, 10, 13, 5, 7,
			10, 1 };
	private static final int[] doubleLeftArrow1 = { 13, 1, 11, 1, 7, 7, 11,
		13, 13, 13, 9, 7 };
	private static final int[] doubleLeftArrow2 = { 8, 1, 6, 1, 3, 7, 6, 13,
		8, 13, 5, 7 };
	// Triangles
	private static final int[] smallLeftTriangle = { 4, 4, 9, 6, 4, 9 };
	private static final int[] largeLeftTriangle = { 11, 1, 1, 6, 11, 11 };

	// Determine the position scaling required
	private static final int MINSIZE = 13;

	/**
	 * This method returns a set of the points required to render a polygon,
	 * this is centered within a provided line height. Shapes will have a
	 * maximum width of 14 pixels.
	 * 
	 * @param lineHeight
	 *            the height of the line the shape is going to be rendered on.
	 * @param shapeType
	 *            the defined enum of the shape.
	 * @param dir
	 * 			  the defined directional enum of this shape.
	 * @return a set that holds all of the shape components, or null if the
	 *         shape could not be found.
	 */
	public static Set<int[]> getShape(int lineHeight, ShapeType type, 
			ShapeDirection dir) {
		int increment = (lineHeight - MINSIZE)/2 - 3;
		Set<int[]> shapeSet = new HashSet<int[]>();
		// General Shapes
		if (type.equals(ShapeType.star))
			shapeSet.add(star.clone());
		else if (type.equals(ShapeType.smallsquare))
			shapeSet.add(smallSquare.clone());
		else if (type.equals(ShapeType.largesquare))
			shapeSet.add(largeSquare.clone());
		// Arrows
		else if (type.equals(ShapeType.arrow))
			shapeSet.add(leftArrow.clone());
		else if (type.equals(ShapeType.doublearrow)) {
			shapeSet.add(doubleLeftArrow1.clone());
			shapeSet.add(doubleLeftArrow2.clone());
			// Triangles
		} else if (type.equals(ShapeType.smalltriangle))
			shapeSet.add(smallLeftTriangle.clone());
		else if (type.equals(ShapeType.largetriangle))
			shapeSet.add(largeLeftTriangle.clone());
		else
			return null;
		rotateShape(shapeSet, dir, lineHeight);
		// Deal with different line heights
		Set<int[]> finalSet = new HashSet<int[]>();
		for (int[] arr : shapeSet) {
			int[] curArr = new int[arr.length];
			for (int i = 0; i < arr.length; i++) {
				if (i % 2 == 1)
					curArr[i] = arr[i] + increment;
				else
					curArr[i] = arr[i];
			}
			finalSet.add(curArr);
		}
		return finalSet;
	}
	
	private static void rotateShape(Set<int[]> shape, ShapeDirection dir,
			int lineHeight){
		int numRots = dir.ordinal();
		for(int i = 1; i <= numRots; i++){
			rotate90(shape, lineHeight);
		}
		
		
	}

	private static void rotate90(Set<int[]> shape, int lineHeight) {
		for(int[] arr : shape){
			for(int b = 0; b < arr.length; b += 2){
				int x = arr[b];
				int y = arr[b+1];
				//always going to be positive coordinates...
				y = -y;
				//Reverse values
				arr[b] = y + MINSIZE;
				arr[b+1] = x;
			}
		}
	}
}