package metricsOverlay.model.mappings;

import java.util.Map;

import metricsOverlay.model.IEditable;
import metricsOverlay.model.INameable;
import metricsOverlay.model.augmentation.Augmentation;
import metricsOverlay.model.metrics.Metric;
import metricsOverlay.model.metrics.Range;

/**
 * This class provides a model implementation of a mapping. It holds two
 * components, a metric and a augmentation and contains methods for
 * interpolating between the two disjunct concepts.
 * 
 * @author Matthew Harward
 * @version 2/9/2009
 */
public class Mapping implements Comparable<Mapping>, INameable, IEditable {

	// Private Fields - Main
	private String name;
	private Metric metric;
	private Augmentation augmentation;

	// Private Fields - State
	private MappingState state;
	private boolean editable;

	// Interpolation Related Fields
	private boolean interpolationReady;
	private InterpolationStrategy interStrat;
	private RangeInterpolator rangeInterpol;
	private AugmentationInterpolator augInterpol;

	// Deprecated
	@Deprecated
	private MappingProperties mProperties;

	/**
	 * Main constructor in initial development. Now deprecated.
	 * 
	 * @param mapName
	 *            the name of the mapping.
	 * @param augName
	 *            the standard name for the augmentation
	 * @param properties
	 *            a map describing a set of properties.
	 */
	@Deprecated
	public Mapping(String mapName, String augName,
			Map<String, String> properties) {
		// setMetricSections(new HashSet<MetricSection>());
		mProperties = new MappingProperties(properties);
		state = MappingState.ActiveMapping;
		name = mapName;
		editable = true;
		// TEMP
		// augmentation = new FullLineAugmentation(displayName,
		// AugmentationType.TwoColouring);
	}

	/**
	 * Basic constructor, can be used to create a stub mapping when little
	 * information on the mapping is know.
	 * 
	 * @param mappingName
	 *            the name of the mapping.
	 * @param stubMapping
	 *            whether the mapping is a stub or not.
	 */
	public Mapping(String mappingName, boolean stubMapping) {
		this(mappingName);
		if (stubMapping)
			state = MappingState.StubMapping;
	}

	/**
	 * Basic constructor.
	 * 
	 * @param name
	 *            the name of the mapping.
	 */
	public Mapping(String name) {
		state = MappingState.UndeclaredMapping;
		this.name = name;
		editable = true;
		interpolationReady = false;
	}

	/**
	 * Full Constructor.
	 * 
	 * @param name
	 *            the name of the mapping.
	 */
	public Mapping(String name, Metric met, Augmentation aug,
			InterpolationStrategy iStrat, MappingState state, boolean editable) {
		this(name);
		metric = met;
		augmentation = aug;
		interStrat = iStrat;
		this.state = state;
		this.editable = editable;
	}

	/**
	 * This method takes whatever value has been provided to a metrics section
	 * and turns it into a value for use by the augmentation algorithm.
	 * 
	 * @param value
	 *            the value stored by the metric section
	 * @return a interpolated value
	 */
	public int getInterpolatedValue(String value) {
		if (!interpolationReady) {
			if (interStrat == null)
				interStrat = new LinearIStrategy();
			restartInterpolationStrategy();
		}
		// Interpolate in 3 stages...
		// Over range
		double inter = rangeInterpol.interpolateValue(value);
		// Over mapping
		int inter2 = interStrat.getInterpolatedValue(String.valueOf(inter));
		// Over augmentation settings
		return augInterpol.interpolateValue(inter2);
	}

	/**
	 * This method sets up the interpolation conditions.
	 */
	private void restartInterpolationStrategy() {
		// Intial checks
		if (state.equals(MappingState.StubMapping) || !isUsable()) {
			return;
		}
		// Set up values
		Range curRange = getMetric().getRange();
		rangeInterpol = new RangeInterpolator(curRange);
		Augmentation dStrat = getAugmentation();
		augInterpol = new AugmentationInterpolator(dStrat);
		double dMax = augInterpol.getMax();
		double dMin = augInterpol.getMin();
		double mMax = rangeInterpol.getMax();
		double mMin = rangeInterpol.getMin();
		interStrat.setMinMax(dMax, dMin, mMax, mMin);
		interpolationReady = true;
	}

	// General Getters/Setters

	/**
	 * Gets the set name of the mapping.
	 */
	public String getName() {
		return name;
	}

	/**
	 * Sets the name of the mapping.
	 */
	public void setName(String name) {
		this.name = name;
	}

	// Get and set the state.

	/**
	 * Gets the current state of this mapping.
	 */
	public MappingState getMappingState() {
		return state;
	}

	/**
	 * Sets the current state of this mapping.
	 * 
	 * @param state
	 *            the new state.
	 */
	public void setMappingState(MappingState state) {
		this.state = state;
	}

	// Metric and Augmentation Getters/Setters

	/**
	 * Returns the metric currently belonging to this mapping.
	 * 
	 * @return the metric in use.
	 */
	public Metric getMetric() {
		return metric;
	}

	/**
	 * Sets the metric to be used by this mapping algorithm.
	 * 
	 * @param metric
	 *            the metric to be used.
	 */
	public void setMetric(Metric metric) {
		interpolationReady = false;
		this.metric = metric;
	}

	/**
	 * Clears an existing metric from this mapping.
	 */
	public void clearMetric() {
		interpolationReady = false;
		metric = null;
	}

	/**
	 * Gets the current augmentation that is in use with this mapping. If no
	 * augmentation exists, this will return null.
	 * 
	 * @return If it exists, the current augmentation, otherwise null.
	 */
	public Augmentation getAugmentation() {
		return augmentation;
	}

	/**
	 * Sets a particular augmentation to use with this mapping.
	 * 
	 * @param augmentation
	 *            the augmentation to use.
	 */
	public void setAugmentation(Augmentation augmentation) {
		interpolationReady = false;
		this.augmentation = augmentation;
	}

	/**
	 * Clears an existing augmentation from this mapping.
	 */
	public void clearAugmentation() {
		interpolationReady = false;
		augmentation = null;
	}

	/**
	 * Gets the currently used interpolation strategy. Note, if an interpolation
	 * strategy does not exist a <code>
	 * LinearIStrategy</code> will be created by
	 * default.
	 * 
	 * @return the current interpolation strategy.
	 */
	public InterpolationStrategy getInterpolationType() {
		if (interStrat == null)
			interStrat = new LinearIStrategy();
		return interStrat;
	}

	/**
	 * Sets a new interpolation strategy.
	 * 
	 * @param is
	 *            the strategy to use.
	 */
	public void setInterpolationType(InterpolationStrategy is) {
		interStrat = is;
		interpolationReady = false;
	}

	@Override
	public boolean canChangeEditableStatus() {
		return true;
	}

	@Override
	public boolean canEdit() {
		return editable;
	}

	@Override
	public void setEditable(boolean editable) {
		this.editable = editable;
	}

	/**
	 * Determines if this metric is usable. A mapping is considered to be usable
	 * if it has a metric and an augmentation.
	 * 
	 * @return <code>true</code> if usable, else <code>false</code>.
	 */
	public boolean isUsable() {
		if (metric != null && augmentation != null)
			return true;
		else
			return false;
	}

	@Override
	public int compareTo(Mapping o) {
		if (metric.equals(o.metric) && augmentation.equals(o.augmentation)
				&& state == o.state)
			return 1;
		else
			return -1;
	}

	/**
	 * This toString method provides details of the mapping and the metric and
	 * augmentation it is comprised of.
	 */
	public String toString() {
		return "Mapping: " + name + " [From: " + metric.toString() + "] [To: "
				+ augmentation.toString() + "]";
	}

	/**
	 * Gets the value of a property given a property name.
	 * 
	 * @param name
	 *            the name of the property to be located
	 * @return the value of the property, or null if it could not be located.
	 */
	@Deprecated
	public String getPropertyValue(String name) {
		return mProperties.getProperty(name);
	}

	/**
	 * Adds a specific property name and value to the property table of this
	 * mapping.
	 * 
	 * @param name
	 *            the name of the property
	 * @param value
	 *            the value of the property
	 */
	@Deprecated
	public void addPropertyValue(String name, String value) {
		mProperties.addProperty(name, value);
	}

	/**
	 * Returns a modifiable map of properties for this mapping.
	 * 
	 * @return a map of the properties.
	 */
	@Deprecated
	public Map<String, String> getProperties() {
		return mProperties.getAllProperties();
	}
}
