package metricsOverlay.model.mappings;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import java.util.Map;

import metricsOverlay.model.metrics.CompositeIntegerRange;
import metricsOverlay.model.metrics.CompositeRealRange;
import metricsOverlay.model.metrics.IntegerRange;
import metricsOverlay.model.metrics.MappedRange;
import metricsOverlay.model.metrics.OrderedNominalRange;
import metricsOverlay.model.metrics.Range;
import metricsOverlay.model.metrics.RealRange;

/**
 * This class provides the ability to interpolate any range value into a
 * sensible value required by the mapping interpolator.
 * 
 * This class requires knowledge of the different types of ranges that exist.
 * 
 * <p>
 * Shrinking Mapped Ranges is achieved by converting the map into an integer
 * scale where each range/integer in the map is converted to a sequential
 * sequence of map points. (e.g. Good[1,10], Bad[11-20], Ok[21-1000] gets
 * converted to 0,1,2)
 * </p>
 * 
 * This class may be subclassed if needed. All methods should be overridden
 * and super called.
 * 
 * @author Matthew Harward
 * @version 2/9/2009
 */
public class RangeInterpolator {

	protected Range curRange;

	/**
	 * Basic constructor.
	 * 
	 * @param curRange
	 *            the range to be interpolated.
	 */
	public RangeInterpolator(Range curRange) {
		this.curRange = curRange;
	}

	/**
	 * Gets the max value of any range types. Note this method shrinks any
	 * mapped metric into an integer range.
	 * 
	 * @return a double value representing the interpolation.
	 */
	public double getMax() {
		if (curRange instanceof MappedRange) {
			return ((MappedRange) curRange).getNumItems() - 1;
		} else
			return Double.valueOf(curRange.getMax());
	}

	/**
	 * Gets the min value of any range types. Note this method shrinks any
	 * mapped metric into an integer range.
	 * 
	 * @return a double value representing the interpolation.
	 */
	public double getMin() {
		if (curRange instanceof MappedRange) {
			return ((MappedRange) curRange).getNumItems() - 1;
		} else
			return Double.valueOf(curRange.getMin());
	}

	/**
	 * This method interpolates values dependent on the type of range they
	 * belong to.
	 * 
	 * @param value
	 *            a String value representing the value to be interpolated.
	 * @return a value that can be used by mapping.
	 */
	public double interpolateValue(String value) {
		Double finalVal = null;
		if (curRange instanceof IntegerRange || curRange instanceof RealRange) {
			finalVal = simpleRangeInterpolator(value);
		} else if (curRange instanceof OrderedNominalRange) {
			finalVal = orderedNominalRangeInterpolator(value);
		} else if (curRange instanceof CompositeIntegerRange) {
			finalVal = compositeIntegerRangeInterpolator(value);
		} else if (curRange instanceof CompositeRealRange) {
			finalVal = compositeRealRangeInterpolator(value);
		}
		return finalVal;
	}

	/**
	 * Interpolation for <code>IntegerRange</code> and <code>RealRange</code>.
	 */
	private Double simpleRangeInterpolator(String value) {
		Double finalVal;
		Double dblVal = Double.valueOf(value);
		Double max = Double.valueOf(curRange.getMax());
		Double min = Double.valueOf(curRange.getMin());
		assert (max >= min);
		if (max < dblVal)
			dblVal = max;
		if (min > dblVal)
			dblVal = min;
		finalVal = dblVal;
		return finalVal;
	}

	/**
	 * Interpolation for an <code>OrderedNominalRange</code>.
	 */
	private Double orderedNominalRangeInterpolator(String value) {
		Map<Integer, String> onMap = ((OrderedNominalRange) curRange)
				.getRangeMap();
		Integer chosenInt = null;
		if (onMap.containsKey(Integer.valueOf(value))) {
			chosenInt = Integer.valueOf(value);
		} else if (onMap.containsValue(value)) {
			for (Integer i : onMap.keySet()) {
				if (onMap.get(i).equals(value)) {
					chosenInt = i;
					break;
				}
			}
		}
		List<Integer> vals = new ArrayList<Integer>(onMap.keySet());
		Collections.sort(vals);
		int returnInt = vals.indexOf(chosenInt);
		return Double.valueOf(Integer.toString(returnInt));
	}

	/**
	 * Interpolation for an <code>CompositeIntegerRange</code>.
	 */
	private Double compositeIntegerRangeInterpolator(String value) {
		Map<IntegerRange, String> onMap = ((CompositeIntegerRange) curRange)
				.getRangeMap();
		IntegerRange chosenIntRange = null;
		for (IntegerRange i : onMap.keySet()) {
			if (onMap.get(i).equals(value)) {
				chosenIntRange = i;
			}
			if (i.isInRange(value)) {
				chosenIntRange = i;
				break;
			}
		}
		ArrayList<IntegerRange> vals = new ArrayList<IntegerRange>(onMap
				.keySet());
		Collections.sort(vals, getIntRangeComparable());
		int returnInt = vals.indexOf(chosenIntRange);
		return Double.valueOf(Integer.toString(returnInt));
	}

	/**
	 * A simple <code>IntegerRange</code> comparator generator. Two range items
	 * are compared by the total size of their maximum and minimum values. As
	 * ranges may not overlap, this produces an accurate comparator.
	 * 
	 * @return the generated comparator
	 */
	private Comparator<IntegerRange> getIntRangeComparable() {
		return new Comparator<IntegerRange>() {

			@Override
			public int compare(IntegerRange intR1, IntegerRange intR2) {
				int r1Tot = Integer.valueOf(intR1.getMax())
						+ Integer.valueOf(intR1.getMin());
				int r2Tot = Integer.valueOf(intR2.getMax())
						+ Integer.valueOf(intR2.getMin());
				if (r1Tot > r2Tot)
					return 1;
				else if (r2Tot > r1Tot)
					return -1;
				else
					return 0;
			}
		};
	}

	/**
	 * Interpolation for an <code>CompositeRealRange</code>.
	 */
	private Double compositeRealRangeInterpolator(String value) {
		Map<RealRange, String> onMap = ((CompositeRealRange) curRange)
				.getRangeMap();
		RealRange chosenRealRange = null;
		for (RealRange i : onMap.keySet()) {
			if (onMap.get(i).equals(value)) {
				chosenRealRange = i;
			}
			if (i.isInRange(value)) {
				chosenRealRange = i;
				break;
			}
		}
		ArrayList<RealRange> vals = new ArrayList<RealRange>(onMap.keySet());
		Collections.sort(vals, getRealRangeComparable());
		int returnInt = vals.indexOf(chosenRealRange);
		return Double.valueOf(Integer.toString(returnInt));
	}

	/**
	 * A simple <code>RealRange</code> comparator generator. Two range items are
	 * compared by the total size of their maximum and minimum values. As ranges
	 * may not overlap, this produces an accurate comparator.
	 * 
	 * @return the generated comparator
	 */
	private Comparator<RealRange> getRealRangeComparable() {
		return new Comparator<RealRange>() {

			@Override
			public int compare(RealRange realR1, RealRange realR2) {
				double r1Tot = Double.valueOf(realR1.getMax())
						+ Double.valueOf(realR1.getMin());
				double r2Tot = Double.valueOf(realR2.getMax())
						+ Double.valueOf(realR2.getMin());
				if (r1Tot > r2Tot)
					return 1;
				else if (r2Tot > r1Tot)
					return -1;
				else
					return 0;
			}
		};
	}

}
