package metricsOverlay.preferences;

import java.util.HashSet;
import java.util.Set;

import metricsOverlay.Activator;
import metricsOverlay.handlers.MetricFile;
import metricsOverlay.handlers.MetricFileHandler;
import metricsOverlay.model.augmentation.MarginAugmentation;
import metricsOverlay.model.augmentation.UnderlineAugmentation;

import org.eclipse.jface.preference.BooleanFieldEditor;
import org.eclipse.jface.preference.FieldEditor;
import org.eclipse.jface.preference.FieldEditorPreferencePage;
import org.eclipse.jface.preference.ScaleFieldEditor;
import org.eclipse.jface.preference.StringFieldEditor;
import org.eclipse.jface.resource.FontDescriptor;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.StackLayout;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPreferencePage;

/**
 * This provides the code in-front for the augmentation
 * preference page.
 * 
 * @author Matthew Harward
 * @version 26/8/2009
 */
public class AugmentationPreferencePage
	extends FieldEditorPreferencePage
	implements IWorkbenchPreferencePage {

	public static final String AUG_NAME = "mOverlay.aug.name";
	public static final String AUG_TYPE = "mOverlay.aug.type";
	public static final String AUG_IMPORT = "mOverlay.aug.importance";
	public static final String AUG_ANNOTATE = "mOverlay.aug.annotations";
	public static final String AUG_DISCRETE = "mOverlay.aug.discrete";
	public static final String AUG_COLOR_EDIT = "mOverlay.aug.coloreditor";
	
	private Set<FieldEditor> fEditors;
	private Combo selector;
	private Group gAdd;
	private Combo typeSelector;
	private AugmentationPrefController aCont;
	private Composite specialistGroup;
	private StackLayout specialistLayout;
	private HashSet<Composite> specialComposites;
	private Combo underlineType;
	private Combo shapeTypeCombo;
	private Combo shapeDirCombo;
	private ColorListEditor clEdit;
	private Combo marginPosition;
	private Label editable;
	private Button update;
	private Button del;

	public Set<FieldEditor> getFieldEditors() {
		return fEditors;
	}

	public Combo getAugSelector() {
		return selector;
	}
	
	public Combo getTypeSelector() {
		return typeSelector;
	}

	public AugmentationPreferencePage() {
		super(GRID);
		setTitle("Metrics Overlay - Augmentation Preferences");
		setDescription("Select an augmentation type and change your preferences.");
		setPreferenceStore(Activator.getDefault().getPreferenceStore());
		fEditors = new HashSet<FieldEditor>();
		noDefaultAndApplyButton();
	}
	
	public FieldEditor getFieldEditor(String editor){
		for(FieldEditor fEdit : fEditors){
			if(fEdit.getPreferenceName().equals(editor)){
				return fEdit;
			}
		}
		return null;
	}
	
	/**
	 * Creates the field editors. Field editors are abstractions of
	 * the common GUI blocks needed to manipulate various types
	 * of preferences. Each field editor knows how to save and
	 * restore itself.
	 */
	public void createFieldEditors() {
		
		MetricFile mf = MetricFileHandler.getInstance().getCurrentMFile();
		aCont = new AugmentationPrefController(mf, this);
		
		Label title = new Label(getFieldEditorParent(), SWT.NONE);
		title.setText("Select Augmentation");
		selector = new Combo(getFieldEditorParent(), SWT.READ_ONLY);
		selector.setItems(aCont.getAugNames());
		selector.addSelectionListener(aCont.getComboListener());
		gAdd = new Group(getFieldEditorParent(),SWT.NONE);
		GridData gData = new GridData(SWT.NONE,SWT.FILL,true,false);
		gData.horizontalSpan = 2;
		gAdd.setLayoutData(gData);
		gAdd.setText("Edit/Add Augmentation");
		
		editable = new Label(gAdd,SWT.NONE);
		GridData editData = new GridData();
		editData.horizontalSpan = 2;
		editable.setLayoutData(editData);
		FontDescriptor fd = FontDescriptor.createFrom(editable.getFont());
		FontDescriptor fd2 = fd.setStyle(SWT.ITALIC);
		editable.setFont(fd2.createFont(null));
		
		StringFieldEditor nameField = new StringFieldEditor(AUG_NAME,"Augmentation Name",40,gAdd);
		addAField(nameField);
		
		Label typeLabel = new Label(gAdd, SWT.NONE);
		typeLabel.setText("Augmentation Type");
		typeSelector = new Combo(gAdd, SWT.READ_ONLY);
		typeSelector.setItems(aCont.getAugTypes());
		typeSelector.addSelectionListener(aCont.getTypeComboListener());
		typeSelector.select(0);
		
		ScaleFieldEditor importField = new ScaleFieldEditor(AUG_IMPORT, "Importance\n(Least to Most)", gAdd,
				0,99,1,20);
		addAField(importField);
		BooleanFieldEditor annField = new BooleanFieldEditor(AUG_ANNOTATE,"Toggle annotations on/off", gAdd);
		addAField(annField);
		BooleanFieldEditor disField = new BooleanFieldEditor(AUG_DISCRETE,"Toggle on for discrete colouring, off for continuous", gAdd);
		addAField(disField);
		addField(new SpacerFieldEditor(gAdd));
		clEdit = new ColorListEditor(AUG_COLOR_EDIT,"Colour Editor", gAdd);
		addAField(clEdit);
		
		//Set specialist fields
		specialistGroup = new Composite(gAdd, SWT.NONE);
		GridData specialD = new GridData();
		specialD.horizontalSpan = 2;
		specialistGroup.setLayoutData(specialD);
		specialistLayout = new StackLayout();
		specialistGroup.setLayout(specialistLayout);
		specialComposites = new HashSet<Composite>();
		
		final Composite margin = new Composite(specialistGroup, SWT.NONE);
		GridData gData2 = new GridData();
		gData2.horizontalSpan = 2;
		margin.setLayoutData(gData2);
		GridLayout r2 = new GridLayout();
		r2.numColumns = 2;
		margin.setLayout(r2);
		Label shapeType = new Label(margin,SWT.NONE);
		shapeType.setText("Shape Type");
		shapeTypeCombo = new Combo(margin,SWT.READ_ONLY);
		shapeTypeCombo.setItems(aCont.getShapeNames());
		shapeTypeCombo.select(0);
		Label shapeDir = new Label(margin,SWT.NONE);
		shapeDir.setText("Shape Direction");
		shapeDirCombo = new Combo(margin,SWT.READ_ONLY);
		shapeDirCombo.setItems(aCont.getDirectionNames());
		shapeDirCombo.select(0);
		Label position = new Label(margin, SWT.NONE);
		position.setText("Margin Side");
		marginPosition = new Combo(margin,SWT.READ_ONLY);
		marginPosition.setItems(new String[] {"Left","Right"});
		marginPosition.select(0);
		margin.setData(new MarginAugmentation().getType());
		specialComposites.add(margin);
		
		final Composite underline = new Composite(specialistGroup, SWT.NONE);
		Label underlineName = new Label(underline,SWT.NONE);
		underlineName.setText("Underline Type");
		underlineType = new Combo(underline,SWT.READ_ONLY);
		underlineType.setItems(aCont.getUnderlineType());
		underlineType.select(0);
		GridData gData3 = new GridData();
		gData3.horizontalSpan = 2;
		underline.setLayoutData(gData3);
		GridLayout r3 = new GridLayout();
		r3.numColumns = 2;
		underline.setLayout(r3);
		underline.setData(new UnderlineAugmentation().getType());
		specialComposites.add(underline);
		
		final Composite none = new Composite(specialistGroup, SWT.NONE);
		specialComposites.add(none);
		
		adjustGridLayout();
	}
	
	protected void addAField(FieldEditor fEdit){
		fEditors.add(fEdit);
		addField(fEdit);
	}
	
	protected void contributeButtons(Composite parent) {
		// Setup new layout
		GridLayout layout = new GridLayout();
		layout.numColumns = 4;
		layout.makeColumnsEqualWidth = true;
		parent.setLayout(layout);
		// Add Buttons
		Button add = new Button(parent, SWT.NONE);
		add.setText("Add New...");
		add.setToolTipText("Add a new augmentation with the current setup.");
		add.addSelectionListener(aCont.getAddListener());
		add.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL));
		update = new Button(parent, SWT.NONE);
		update.setText("Update Selected");
		update.setToolTipText("Update the current augmentation.");
		update.addSelectionListener(aCont.getUpdateListener());
		update.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL));
		del = new Button(parent, SWT.NONE);
		del.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL));
		del.setText("Remove Selected");
		del.setToolTipText("Remove the current augmentation");
		del.addSelectionListener(aCont.getDeleteListener());
		Button reset = new Button(parent, SWT.NONE);
		reset.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL));
		reset.setText("Reset");
		reset.setToolTipText("Reset this page");
		reset.addSelectionListener(aCont.getResetListener());
	}
	
	protected void setEditable(boolean yes){
		if(yes){
			editable.setText("This augmentation may be edited");
		}
		else{
			editable.setText("This augmentation may not be edited");
		}
		update.setEnabled(yes);
		del.setEnabled(yes);
		editable.pack();
	}
	
	protected void setSpeciality(String spec){
		Composite set = null;
		Composite none = null;
		for(Composite com : specialComposites){
			if(com.getData() != null && ((String)com.getData()).equals(spec)){
				set = com;
			}
			if(com.getData() == null){
				none = com;
			}
		}
		if(set == null){
			set = none;
		}
		specialistLayout.topControl = set;
		specialistGroup.layout();
	}
	

	/* (non-Javadoc)
	 * @see org.eclipse.ui.IWorkbenchPreferencePage#init(org.eclipse.ui.IWorkbench)
	 */
	public void init(IWorkbench workbench) {
	}
	
	public AugmentationPrefController getController(){
		return aCont;
	}

	
	class StackLayoutSelectionAdapter extends SelectionAdapter {
		Composite shell;
		  StackLayout layout;

		  public StackLayoutSelectionAdapter(Composite shell, StackLayout layout) {
		    this.shell = shell;
		    this.layout = layout;
		  }

		  public void widgetSelected(SelectionEvent event) {
		   // Control control = layout.topControl;
		    Control[] children = shell.getChildren();
		    layout.topControl = children[0];
		    shell.layout();
		  }
		}


	public Combo getUnderlineSelector() {
		return underlineType;
	}

	public Combo getShapeDirSelector() {
		return shapeDirCombo;
	}

	public Combo getShapeTypeSelector() {
		return shapeTypeCombo;
	}

	public ColorListEditor getColorEditor() {
		return clEdit;
	}

	public void reset() {
		for(FieldEditor ed : getFieldEditors()){
			ed.loadDefault();
		}
	}

	public Combo getPositionSelector() {
		return marginPosition;
	}
	
}
