package metricsOverlay.preferences;

import metricsOverlay.Activator;
import metricsOverlay.OverlayRegistry;
import metricsOverlay.handlers.MetricFile;
import metricsOverlay.handlers.MetricFileHandler;
import metricsOverlay.model.augmentation.Augmentation;
import metricsOverlay.model.mappings.ExponentialIStrategy;
import metricsOverlay.model.mappings.InterpolationStrategy;
import metricsOverlay.model.mappings.LinearIStrategy;
import metricsOverlay.model.mappings.LogIStrategy;
import metricsOverlay.model.mappings.Mapping;
import metricsOverlay.model.mappings.MappingState;
import metricsOverlay.model.mappings.MetricLogIStrategy;
import metricsOverlay.model.metrics.Metric;

import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.preference.FieldEditor;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.ui.PlatformUI;

public class MappingPrefController {

	private MetricFile mf;
	private MappingPreferencePage mPref;
	private IPreferenceStore store;
	protected Mapping curMap;

	public MappingPrefController(MetricFile mf,
			MappingPreferencePage mPreferencePage) {
		this.mf = mf;
		mPref = mPreferencePage;
		curMap = null;
		store = Activator.getDefault().getPreferenceStore();
	}

	public void setActiveMapping(Mapping m) {
		int a;
		if ((a = mPref.getMapSelector().indexOf(m.getName())) > -1) {
			mPref.getMapSelector().select(a);
			mPref.getMapSelector().setText(m.getName());
			mPref.getMapSelector().update();
			onMapSelection(m.getName());
		}
	}

	public void onMapSelection(String name) {
		curMap = mf.getMetricDocument().getMapping(name);
		
		mPref.setEditable(curMap.canEdit());
		
		store.setValue(MappingPreferencePage.MAP_NAME, curMap.getName());
		store.setValue(MappingPreferencePage.MAP_STATUS, EnumConverter
				.makeReadable(curMap.getMappingState().toString()));
		store.setValue(MappingPreferencePage.MAP_INTERCONST, curMap
				.getInterpolationType().getType());
		mPref.getMetricSelector().select(
				mPref.getMetricSelector().indexOf(curMap.getMetric().getName()));
		mPref.getAugSelector().select(
				mPref.getAugSelector().indexOf(curMap.getAugmentation().getName()));
		mPref.getInterSelector().select(
				mPref.getInterSelector().indexOf(
						curMap.getInterpolationType().getType()));

		// Specialist Strategies
		InterpolationStrategy is = curMap.getInterpolationType();
		if (is instanceof LogIStrategy) {
			store.setValue(MappingPreferencePage.MAP_INTERCONST,
					((LogIStrategy) is).getConstant());
		} else if (is instanceof ExponentialIStrategy) {
			store.setValue(MappingPreferencePage.MAP_INTERCONST,
					((ExponentialIStrategy) is).getConstant());
		}
		else store.setValue(MappingPreferencePage.MAP_INTERCONST, "0");

		if (needsConstant(store
				.getString(MappingPreferencePage.MAP_INTERPOLATE)))
			mPref.setSpeciality(new MetricLogIStrategy().getType());
		else
			mPref.setSpeciality(null);

		// Reload all field editors
		for (FieldEditor edit : mPref.getFieldEditors()) {
			edit.load();
		}
	}

	private Mapping operateOnMapping(Mapping mapper) {
		mapper.setName(store.getString(MappingPreferencePage.MAP_NAME));
		mapper.setMappingState(MappingState.valueOf(EnumConverter
				.makeEnum(store.getString(MappingPreferencePage.MAP_STATUS))));
		mapper.setAugmentation(mf.getMetricDocument().getAugmentation(
				mPref.getAugSelector().getText()));
		mapper.setMetric(mf.getMetricDocument().getMetric(
				mPref.getMetricSelector().getText()));
		InterpolationStrategy is = OverlayRegistry.getInstance()
				.getInterpolation(mPref.getInterSelector().getText());
		if (is instanceof LogIStrategy) {
			((LogIStrategy) is).setConstant(Double
					.parseDouble(MappingPreferencePage.MAP_INTERCONST));
		} else if (is instanceof ExponentialIStrategy) {
			((ExponentialIStrategy) is).setConstant(Double
					.parseDouble(MappingPreferencePage.MAP_INTERCONST));
		}
		mapper.setInterpolationType(is);
		return mapper;
	}

	private void updateSelector(String curSelected) {
		mPref.getMapSelector().setItems(getNamedMappings());
		if (curSelected == null || curSelected == "")
			mPref.getMapSelector().clearSelection();
		else {
			int i = mPref.getMapSelector().indexOf(curSelected);
			mPref.getMapSelector().select(i);
		}
	}

	public String[] getInterNames() {
		return OverlayRegistry.getInstance().getAllInterNames();
	}

	public String[] getNamedMappings() {
		String[] names = new String[0];
		if (mf != null) {
			names = new String[mf.getMetricDocument().getAllMappings().size()];
			int count2 = 0;
			for (Mapping mm : mf.getMetricDocument().getAllMappings()) {
				names[count2++] = mm.getName();
			}
		}
		return names;
	}

	public String[] getNamedMetrics() {
		String[] names = new String[0];
		if (mf != null) {
			names = new String[mf.getMetricDocument().getAllMetrics().size()];
			int count2 = 0;
			for (Metric mm : mf.getMetricDocument().getAllMetrics()) {
				names[count2++] = mm.getName();
			}
		}
		return names;
	}

	public String[] getNamedAugs() {
		String[] names = new String[0];
		if (mf != null) {
			names = new String[mf.getMetricDocument().getAllAugmentations()
					.size()];
			int count2 = 0;
			for (Augmentation mm : mf.getMetricDocument().getAllAugmentations()) {
				names[count2++] = mm.getName();
			}
		}
		return names;
	}

	public String[][] getStatusTypes() {
		MappingState[] at = MappingState.values();
		final String[][] status = new String[at.length - 1][1];
		int count = 0;
		for (MappingState aa : at) {
			if (aa.ordinal() > 0) {
				final String[] row = new String[2];
				row[0] = EnumConverter.makeReadable(aa.name());
				row[1] = row[0];
				status[count++] = row;
			}
		}
		return status;
	}

	public SelectionListener getComboListener() {
		return new SelectionListener() {

			@Override
			public void widgetSelected(SelectionEvent e) {
				String selected = ((Combo) e.widget).getItem(((Combo) e.widget)
						.getSelectionIndex());
				onMapSelection(selected);
			}

			@Override
			public void widgetDefaultSelected(SelectionEvent e) {
			}
		};
	}

	public SelectionListener getInterpolateComboListener() {
		return new SelectionListener() {

			@Override
			public void widgetSelected(SelectionEvent e) {
				String selected = ((Combo) e.widget).getItem(((Combo) e.widget)
						.getSelectionIndex());
				store.putValue(MappingPreferencePage.MAP_INTERPOLATE, selected);
				if (needsConstant(store
						.getString(MappingPreferencePage.MAP_INTERPOLATE)))
					mPref.setSpeciality(new MetricLogIStrategy().getType());
				else
					mPref.setSpeciality(null);
			}

			@Override
			public void widgetDefaultSelected(SelectionEvent e) {
			}

		};
	}

	private boolean needsConstant(String string) {
		boolean interable = true;
		if (new LinearIStrategy().getType().equals(string))
			interable = false;
		return interable;
	}

	public SelectionListener getAddListener() {
		return new SelectionListener() {

			@Override
			public void widgetSelected(SelectionEvent e) {
				if(curMap != null){
					storeAll();
					String theName = store.getString(MappingPreferencePage.MAP_NAME);
					// Check for uniqueness...
					if (!mf.getMetricDocument().isMappingNameUnique(theName)) {
						queryDialog("Duplicate Mapping", "You are trying" +
								" to create an mapping with a name that already exists."
								, MessageDialog.ERROR);
					} else {
						newMapping();
					}
				}
			}

			@Override
			public void widgetDefaultSelected(SelectionEvent e) {
			}
		};
	}

	public SelectionListener getUpdateListener() {
		return new SelectionListener() {

			@Override
			public void widgetSelected(SelectionEvent e) {
				if(curMap != null){
					storeAll();
					String theName = store.getString(MappingPreferencePage.MAP_NAME);
					//If the name hasn't changed
					if(curMap.getName().equals(theName)){
						updateMapping();
					}
					// If the name is unique, but changed
					else if (mf.getMetricDocument().isAugmentationNameUnique(theName)) {
						if(queryDialog("Name Changed", "You have changed the name of the" +
								" mapping from \"" + curMap.getName() + "\" to \"" + theName +
								"\".\nDo you still want to update?", MessageDialog.QUESTION)){
							updateMapping();
						}
					} else {
						queryDialog("Name Conflict", "You have changed the name of the" +
								" mapping to one that is not unique.\n Please change"
								+" it if you wish to update.", MessageDialog.WARNING);
					}
				}
			}

			@Override
			public void widgetDefaultSelected(SelectionEvent e) {
			}
		};
	}

	public SelectionListener getDeleteListener() {
		return new SelectionListener() {

			@Override
			public void widgetSelected(SelectionEvent e) {
				if(curMap != null){
					storeAll();
					String theName = store.getString(MappingPreferencePage.MAP_NAME);
					//If the name to the current augmentation has not been changed.
					if(curMap.getName().equals(theName))
					{
						if(queryDialog("Delete Confirmation","Are you sure you wish to delete this mapping?",
								MessageDialog.QUESTION)){
							deleteMapping();
						}
					}
					// Check for uniqueness... If not unique
					else if (!mf.getMetricDocument().isMappingNameUnique(theName)) {
						queryDialog("Delete Problem","You have changed the name of this mapping "+
								"to \nthat of another. Please change it to something unique.",
								MessageDialog.ERROR);
					} else {
						if(queryDialog("Delete Confirmation","You have changed the name of this mapping from \""+
								curMap.getName() + "\" to \"" + theName + "\".\n" + 
								"Are you sure you still wish to delete it?",
								MessageDialog.QUESTION)){
							deleteMapping();
						}
					}
					}
			}

			@Override
			public void widgetDefaultSelected(SelectionEvent e) {
			}
		};
	}

	public SelectionListener getResetListener() {
		return new SelectionListener() {

			@Override
			public void widgetSelected(SelectionEvent e) {
				resetPage();
			}

			@Override
			public void widgetDefaultSelected(SelectionEvent e) {
			}
		};
	}
	
	private void newMapping() {
		Mapping mapper = new Mapping(store
				.getString(MappingPreferencePage.MAP_NAME));
		curMap = operateOnMapping(mapper);
	
		mf.getMetricDocument().addMapping(curMap);
		// Then reload page...
		updateSelector(curMap.getName());
		setActiveMapping(curMap);
		MetricFileHandler.getInstance().update(mf,true);
		// Current page will stay loaded into the controls...
	}

	private void updateMapping() {
		curMap = mf.getMetricDocument().getMapping(
				store.getString(MappingPreferencePage.MAP_NAME));
		// Call updates..
		curMap = operateOnMapping(curMap);
		setActiveMapping(curMap);
		MetricFileHandler.getInstance().update(mf,true);
	}

	/**
	 * 
	 */
	private void deleteMapping() {
		mf.getMetricDocument().removeMapping(
				store.getString(MappingPreferencePage.MAP_NAME));
		MetricFileHandler.getInstance().update(mf,true);
		resetPage();
	}

	private void storeAll() {
		for (FieldEditor cur : mPref.getFieldEditors()) {
			cur.store();
		}
	}

	/**
	 * 
	 */
	private boolean queryDialog(String title, String contents, int type) {
		return MessageDialog.open(type, PlatformUI.getWorkbench()
				.getActiveWorkbenchWindow().getShell(),
				title,contents,SWT.NONE);
	}

	/**
	 * @param store
	 */
	private void resetPage() {
		store.setToDefault(MappingPreferencePage.MAP_NAME);
		store.setToDefault(MappingPreferencePage.MAP_INTERCONST);
		store.setToDefault(MappingPreferencePage.MAP_INTERPOLATE);
		store.setToDefault(MappingPreferencePage.MAP_STATUS);
		updateSelector(null);
		mPref.reset();
	}
}
