package metricsOverlay.preferences;

import java.util.HashSet;
import java.util.Set;

import metricsOverlay.Activator;
import metricsOverlay.handlers.MetricFile;
import metricsOverlay.handlers.MetricFileHandler;
import metricsOverlay.model.mappings.MetricLogIStrategy;

import org.eclipse.jface.preference.FieldEditor;
import org.eclipse.jface.preference.FieldEditorPreferencePage;
import org.eclipse.jface.preference.IntegerFieldEditor;
import org.eclipse.jface.preference.RadioGroupFieldEditor;
import org.eclipse.jface.preference.StringFieldEditor;
import org.eclipse.jface.resource.FontDescriptor;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.StackLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPreferencePage;

/**
 * This provides the code in-front for the mapping preference page.
 * 
 * @author Matthew Harward
 * @version 26/8/2009
 */
public class MappingPreferencePage extends FieldEditorPreferencePage implements
		IWorkbenchPreferencePage {

	public static final String MAP_NAME = "mOverlay.map.name";
	public static final String MAP_STATUS = "mOverlay.map.status";
	public static final String MAP_INTERPOLATE = "mOverlay.map.interpolate";
	public static final String MAP_INTERCONST = "mOverlay.map.interconst";

	private Combo mappingSelector;
	private Combo metricSelector;
	private Combo augSelector;
	private MappingPrefController aCont;
	private Set<FieldEditor> fEditors;
	private Composite specialistGroup;
	private StackLayout specialistLayout;
	private HashSet<Composite> specialComposites;
	private Combo interpolateCombo;
	private Button update;
	private Label editable;
	private Button del;

	public MappingPreferencePage() {
		super(GRID);
		setTitle("Metrics Overlay - Mapping Preferences");
		setDescription("Create or modify a mapping between a metric and a display.");
		// setImageDescriptor(ImageDescriptor.createFromImage(new
		// Image(this.getShell().getDisplay(),"icons/mlogo.jpg")));
		setPreferenceStore(Activator.getDefault().getPreferenceStore());
		fEditors = new HashSet<FieldEditor>();
		noDefaultAndApplyButton();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.ui.IWorkbenchPreferencePage#init(org.eclipse.ui.IWorkbench)
	 */
	public void init(IWorkbench workbench) {
	}

	/**
	 * Creates the field editors. Field editors are abstractions of the common
	 * GUI blocks needed to manipulate various types of preferences. Each field
	 * editor knows how to save and restore itself.
	 */
	public void createFieldEditors() {

		MetricFile mf = MetricFileHandler.getInstance().getCurrentMFile();
		aCont = new MappingPrefController(mf, this);

		Label mSelectTitle = new Label(getFieldEditorParent(), SWT.NONE);
		mSelectTitle.setText("Select Mapping");
		mappingSelector = new Combo(getFieldEditorParent(), SWT.READ_ONLY);
		mappingSelector.setItems(aCont.getNamedMappings());
		mappingSelector.addSelectionListener(aCont.getComboListener());
		Group gAdd = new Group(getFieldEditorParent(), SWT.NONE);
		GridData gData = new GridData(SWT.NONE, SWT.FILL, true, false);
		gData.horizontalSpan = 2;
		gAdd.setLayoutData(gData);
		gAdd.setText("Edit/Add Mapping");
		
		editable = new Label(gAdd,SWT.NONE);
		GridData editData = new GridData();
		editData.horizontalSpan = 2;
		editable.setLayoutData(editData);
		FontDescriptor fd = FontDescriptor.createFrom(editable.getFont());
		FontDescriptor fd2 = fd.setStyle(SWT.ITALIC);
		editable.setFont(fd2.createFont(null));

		addAField(new StringFieldEditor(MAP_NAME, "Mapping Name", 40, gAdd));
		Label metricTitle = new Label(gAdd, SWT.NONE);
		metricTitle.setText("Metric");
		metricSelector = new Combo(gAdd, SWT.READ_ONLY);
		metricSelector.setItems(aCont.getNamedMetrics());
		Label augmentTitle = new Label(gAdd, SWT.NONE);
		augmentTitle.setText("Augmentation");
		augSelector = new Combo(gAdd, SWT.READ_ONLY);
		augSelector.setItems(aCont.getNamedAugs());
		Composite c = new Composite(gAdd, SWT.NONE);
		GridData cgd = new GridData();
		cgd.horizontalSpan = 2;
		c.setLayoutData(cgd);
		addAField(new RadioGroupFieldEditor(MAP_STATUS, "Initial Status", 3,
				aCont.getStatusTypes(), c, false));
		Label interMethod = new Label(gAdd, SWT.NONE);
		interMethod.setText("Interpolation Method");
		interpolateCombo = new Combo(gAdd, SWT.READ_ONLY);
		interpolateCombo.setItems(aCont.getInterNames());
		interpolateCombo.addSelectionListener(aCont
				.getInterpolateComboListener());

		// Set specialist fields
		specialistGroup = new Composite(gAdd, SWT.NONE);
		GridData specialD = new GridData();
		specialD.horizontalSpan = 2;
		specialistGroup.setLayoutData(specialD);
		specialistLayout = new StackLayout();
		specialistGroup.setLayout(specialistLayout);
		specialComposites = new HashSet<Composite>();

		final Composite margin = new Composite(specialistGroup, SWT.NONE);
		GridData gData2 = new GridData();
		gData2.horizontalSpan = 2;
		margin.setLayoutData(gData2);
		GridLayout r2 = new GridLayout();
		r2.numColumns = 2;
		margin.setLayout(r2);
		IntegerFieldEditor ife = new IntegerFieldEditor(MAP_INTERCONST,
				"Interpolation Constant", margin);
		GridData intWid = new GridData(SWT.LEFT, SWT.CENTER, true, false);
		intWid.widthHint = 40;
		ife.getTextControl(margin).setLayoutData(intWid);
		addAField(ife);
		margin.setData(new MetricLogIStrategy().getType());
		specialComposites.add(margin);

		final Composite none = new Composite(specialistGroup, SWT.NONE);
		specialComposites.add(none);

		adjustGridLayout();
	}

	protected void contributeButtons(Composite parent) {
		// Setup new layout
		GridLayout layout = new GridLayout();
		layout.numColumns = 4;
		layout.makeColumnsEqualWidth = true;
		parent.setLayout(layout);
		// Add Buttons
		Button add = new Button(parent, SWT.NONE);
		add.setText("Add New...");
		add.setToolTipText("Add a new mapping with the current setup.");
		add.addSelectionListener(aCont.getAddListener());
		add.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL));
		update = new Button(parent, SWT.NONE);
		update.setText("Update Selected");
		update.setToolTipText("Update the current mapping.");
		update.addSelectionListener(aCont.getUpdateListener());
		update.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL));
		del = new Button(parent, SWT.NONE);
		del.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL));
		del.setText("Remove Selected");
		del.setToolTipText("Remove the current mapping");
		del.addSelectionListener(aCont.getDeleteListener());
		Button reset = new Button(parent, SWT.NONE);
		reset.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL));
		reset.setText("Reset");
		reset.setToolTipText("Reset this page");
		reset.addSelectionListener(aCont.getResetListener());
	}
	
	protected void setEditable(boolean yes){
		if(yes){
			editable.setText("This mapping may be edited");
		}
		else{
			editable.setText("This mapping may not be edited");
		}
		update.setEnabled(yes);
		del.setEnabled(yes);
		editable.pack();
	}

	protected void addAField(FieldEditor fEdit) {
		fEditors.add(fEdit);
		addField(fEdit);
	}

	protected Set<FieldEditor> getFieldEditors() {
		return fEditors;
	}

	public MappingPrefController getController() {
		return aCont;
	}

	public void setSpeciality(String string) {
		Composite set = null;
		Composite none = null;
		for (Composite com : specialComposites) {
			if (com.getData() != null
					&& ((String) com.getData()).equals(string)) {
				set = com;
			}
			if (com.getData() == null) {
				none = com;
			}
		}
		if (set == null) {
			set = none;
		}
		specialistLayout.topControl = set;
		specialistGroup.layout();
	}

	public void reset() {
		for (FieldEditor fe : getFieldEditors()) {
			fe.loadDefault();
		}
	}

	public Combo getMapSelector() {
		return mappingSelector;
	}

	public Combo getMetricSelector() {
		return metricSelector;
	}

	public Combo getAugSelector() {
		return augSelector;
	}

	public Combo getInterSelector() {
		return interpolateCombo;
	}
}
