package metricsOverlay.preferences;

import java.util.HashSet;
import java.util.Set;

import metricsOverlay.Activator;
import metricsOverlay.handlers.MetricFile;
import metricsOverlay.handlers.MetricFileHandler;
import metricsOverlay.model.metrics.CompositeIntegerRange;
import metricsOverlay.model.metrics.IntegerRange;
import metricsOverlay.model.metrics.Metric;
import metricsOverlay.model.metrics.OrderedNominalRange;
import metricsOverlay.model.metrics.RealRange;

import org.eclipse.jface.preference.FieldEditor;
import org.eclipse.jface.preference.FieldEditorPreferencePage;
import org.eclipse.jface.preference.IntegerFieldEditor;
import org.eclipse.jface.preference.StringFieldEditor;
import org.eclipse.jface.resource.FontDescriptor;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.StackLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPreferencePage;

/**
 * This provides the code in-front for the metrics
 * preference page.
 * 
 * @author Matthew Harward
 * @version 26/8/2009
 */

public class MetricsPreferencePage
	extends FieldEditorPreferencePage
	implements IWorkbenchPreferencePage {

	public static final String MET_NAME = "mOverlay.met.name";
	public static final String MET_INTMAX = "mOverlay.met.intMax";
	public static final String MET_INTMIN = "mOverlay.met.intMin";
	public static final String MET_REALMIN = "mOverlay.met.realMin";
	public static final String MET_REALMAX = "mOverlay.met.realMax";
	public static final String MET_COMPEDIT = "mOverlay.met.compEdit";
	public static final String MET_ONEDIT = "mOverlay.met.onrEdit";
	
	private MetricsPrefController aCont;
	private Set<FieldEditor> fEditors;
	private Combo selector;
	private Combo derivedSelect;
	private Combo rangeSelect;
	private Label editable;
	private Button update;
	private Composite specialistGroup;
	private StackLayout specialistLayout;
	private HashSet<Composite> specialComposites;
	private Button del;
	private CompositeRangeEditor cre;
	private OrderedNominalEditor onre;


	public MetricsPreferencePage() {
		super(GRID);
		setTitle("Metrics Overlay - Metrics Preferences");
		setDescription("Select a metric and specify your preferences.");
		setPreferenceStore(Activator.getDefault().getPreferenceStore());
		noDefaultAndApplyButton();
		fEditors = new HashSet<FieldEditor>();
	}
	
	/**
	 * Creates the field editors. Field editors are abstractions of
	 * the common GUI blocks needed to manipulate various types
	 * of preferences. Each field editor knows how to save and
	 * restore itself.
	 */
	public void createFieldEditors() {
		
		MetricFile mf = MetricFileHandler.getInstance().getCurrentMFile();
		aCont = new MetricsPrefController(mf, this);
		
		final String[][] contents = new String[30][1];
		contents[0] = new String[] { "Every Day", "1" }; //$NON-NLS-1$//$NON-NLS-2$
		for (int i = 2; i <= 30; i++) {
		        final String[] row = new String[2];
		        row[0] = "Every " + i + "days";
		        row[1] = i + ""; //$NON-NLS-1$
		        contents[i - 1] = row;
		} 
		
		String[][] names = new String[0][0];
		if(mf != null){
		names = new String[mf.getMetricDocument().getAllMetrics().size()][1];
		int count2 = 0;
		for(Metric mm : mf.getMetricDocument().getAllMetrics()){
			final String[] row = new String[2];
			row[0] = mm.getName();
			row[1] = String.valueOf(count2);
			names[count2++] = row;
		} 
		}
		
		Label title = new Label(getFieldEditorParent(), SWT.NONE);
		title.setText("Select Metric");
		selector = new Combo(getFieldEditorParent(), SWT.READ_ONLY);
		selector.setItems(aCont.getMetNames());
		selector.addSelectionListener(aCont.getComboListener());
		
		Group gAdd = new Group(getFieldEditorParent(),SWT.NONE);
		GridData gData = new GridData(SWT.NONE,SWT.FILL,true,false);
		gData.horizontalSpan = 2;
		gAdd.setLayoutData(gData);
		gAdd.setText("Edit/Add Metric");
		
		editable = new Label(gAdd,SWT.NONE);
		GridData editData = new GridData();
		editData.horizontalSpan = 2;
		editable.setLayoutData(editData);
		FontDescriptor fd = FontDescriptor.createFrom(editable.getFont());
		FontDescriptor fd2 = fd.setStyle(SWT.ITALIC);
		editable.setFont(fd2.createFont(null));
		
		addAField(new StringFieldEditor(MET_NAME,"Metric Name",40,gAdd));
		Label derFrom = new Label(gAdd, SWT.NONE);
		derFrom.setText("Derived From");
		derivedSelect = new Combo(gAdd, SWT.READ_ONLY);
		derivedSelect.setItems(aCont.getMetNames());
		derivedSelect.addSelectionListener(aCont.getDerivedComboListener()); //?needed
		Label rangeName = new Label(gAdd, SWT.NONE);
		rangeName.setText("Range Type");
		rangeSelect = new Combo(gAdd, SWT.READ_ONLY);
		rangeSelect.setItems(aCont.getRangeTypes());
		rangeSelect.addSelectionListener(aCont.getRangeComboListener());
		
		// Set specialist fields
		specialistGroup = new Composite(gAdd, SWT.NONE);
		GridData specialD = new GridData();
		specialD.horizontalSpan = 2;
		specialistGroup.setLayoutData(specialD);
		specialistLayout = new StackLayout();
		specialistGroup.setLayout(specialistLayout);
		specialComposites = new HashSet<Composite>();

		final Composite integer = new Composite(specialistGroup, SWT.NONE);
		GridData gData2 = new GridData();
		gData2.horizontalSpan = 2;
		integer.setLayoutData(gData2);
		GridLayout r2 = new GridLayout();
		r2.numColumns = 2;
		integer.setLayout(r2);
		IntegerFieldEditor min = new IntegerFieldEditor(MET_INTMIN,
				"Minimum", integer);
		GridData intMinWid = new GridData(SWT.LEFT, SWT.CENTER, true, false);
		intMinWid.widthHint = 40;
		min.getTextControl(integer).setLayoutData(intMinWid);
		addAField(min);
		IntegerFieldEditor max = new IntegerFieldEditor(MET_INTMAX,
				"Maximum", integer);
		GridData intMaxWid = new GridData(SWT.LEFT, SWT.CENTER, true, false);
		intMaxWid.widthHint = 40;
		max.getTextControl(integer).setLayoutData(intMaxWid);
		addAField(max);
		integer.setData(new IntegerRange().getType());
		specialComposites.add(integer);
		
		final Composite real = new Composite(specialistGroup, SWT.NONE);
		GridData gData3 = new GridData();
		gData3.horizontalSpan = 2;
		real.setLayoutData(gData3);
		GridLayout r3 = new GridLayout();
		r3.numColumns = 2;
		real.setLayout(r3);
		StringFieldEditor minReal = new StringFieldEditor(MET_REALMIN,
				"Minimum", real);
		GridData realMinWid = new GridData(SWT.LEFT, SWT.CENTER, true, false);
		realMinWid.widthHint = 40;
		minReal.getTextControl(real).setLayoutData(realMinWid);
		addAField(minReal);
		StringFieldEditor maxReal = new StringFieldEditor(MET_REALMAX,
				"Maximum", real);
		GridData realMaxWid = new GridData(SWT.LEFT, SWT.CENTER, true, false);
		realMaxWid.widthHint = 40;
		maxReal.getTextControl(real).setLayoutData(realMaxWid);
		addAField(maxReal);
		real.setData(new RealRange().getType());
		specialComposites.add(real);
		
		final Composite onrange = new Composite(specialistGroup, SWT.NONE);
		GridData gData4 = new GridData();
		gData4.horizontalSpan = 2;
		onrange.setLayoutData(gData4);
		GridLayout r4 = new GridLayout();
		r4.numColumns = 2;
		onrange.setLayout(r4);
		onre = new OrderedNominalEditor(MET_ONEDIT,
				"Ordered Nominal Range Editor", onrange);
		addAField(onre);
		onrange.setData(new OrderedNominalRange().getType());
		specialComposites.add(onrange);
		
		final Composite comprange = new Composite(specialistGroup, SWT.NONE);
		GridData gData5 = new GridData();
		gData4.horizontalSpan = 2;
		comprange.setLayoutData(gData5);
		GridLayout r5 = new GridLayout();
		r5.numColumns = 2;
		comprange.setLayout(r5);
		//CompRangeEditor
		cre = new CompositeRangeEditor(MET_COMPEDIT,
				"Composite Range Editor", comprange);
		addAField(cre);
		comprange.setData(new CompositeIntegerRange().getType());
		specialComposites.add(comprange);

		final Composite none = new Composite(specialistGroup, SWT.NONE);
		specialComposites.add(none);
		
		adjustGridLayout();
	}
	
	protected void addAField(FieldEditor fEdit){
		fEditors.add(fEdit);
		addField(fEdit);
	}
	
	protected void contributeButtons(Composite parent) {
		// Setup new layout
		GridLayout layout = new GridLayout();
		layout.numColumns = 4;
		layout.makeColumnsEqualWidth = true;
		parent.setLayout(layout);
		// Add Buttons
		Button add = new Button(parent, SWT.NONE);
		add.setText("Add New...");
		add.setToolTipText("Add a new augmentation with the current setup.");
		add.addSelectionListener(aCont.getAddListener());
		add.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL));
		update = new Button(parent, SWT.NONE);
		update.setText("Update Selected");
		update.setToolTipText("Update the current augmentation.");
		update.addSelectionListener(aCont.getUpdateListener());
		update.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL));
		del = new Button(parent, SWT.NONE);
		del.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL));
		del.setText("Remove Selected");
		del.setToolTipText("Remove the current augmentation");
		del.addSelectionListener(aCont.getDeleteListener());
		Button reset = new Button(parent, SWT.NONE);
		reset.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL));
		reset.setText("Reset");
		reset.setToolTipText("Reset this page");
		reset.addSelectionListener(aCont.getResetListener());
	}
	
	protected void setEditable(boolean yes){
		if(yes){
			editable.setText("This metric may be edited");
		}
		else{
			editable.setText("This metric may not be edited");
		}
		update.setEnabled(yes);
		del.setEnabled(yes);
		editable.pack();
	}
	
	public void setSpeciality(String string) {
		Composite set = null;
		Composite none = null;
		for (Composite com : specialComposites) {
			if (com.getData() != null
					&& ((String) com.getData()).equals(string)) {
				set = com;
			}
			if (com.getData() == null) {
				none = com;
			}
		}
		if (set == null) {
			set = none;
		}
		specialistLayout.topControl = set;
		specialistGroup.layout();
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.IWorkbenchPreferencePage#init(org.eclipse.ui.IWorkbench)
	 */
	public void init(IWorkbench workbench) {
	}

	public Set<FieldEditor> getFieldEditors() {
		return fEditors;
	}

	public MetricsPrefController getController() {
		return aCont;
	}
	
	public CompositeRangeEditor getCompositeRangeEditor(){
		return cre;
	}

	public Combo getMetSelector() {
		return selector;
	}
	
	public Combo getDerivedMetSelector() {
		return derivedSelect;
	}

	public Combo getRangeSelector() {
		return rangeSelect;
	}

	public void reset() {
		for(FieldEditor ed : getFieldEditors()){
			ed.loadDefault();
		}
	}
}
