package metricsOverlay.views;

import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Arrays;
import java.util.List;

import metricsOverlay.handlers.MetricFile;
import metricsOverlay.handlers.MetricFileHandler;
import metricsOverlay.model.MetricDocument;
import metricsOverlay.model.mappings.Mapping;
import metricsOverlay.model.mappings.MappingState;
import metricsOverlay.preferences.AugmentationPreferencePage;
import metricsOverlay.preferences.MappingPreferencePage;
import metricsOverlay.preferences.MetricsPreferencePage;

import org.eclipse.jface.preference.PreferenceDialog;
import org.eclipse.jface.util.IPropertyChangeListener;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.jface.viewers.CellEditor;
import org.eclipse.jface.viewers.CheckboxCellEditor;
import org.eclipse.jface.viewers.ColumnViewerEditor;
import org.eclipse.jface.viewers.ColumnViewerEditorActivationEvent;
import org.eclipse.jface.viewers.ColumnViewerEditorActivationListener;
import org.eclipse.jface.viewers.ColumnViewerEditorActivationStrategy;
import org.eclipse.jface.viewers.ColumnViewerEditorDeactivationEvent;
import org.eclipse.jface.viewers.DialogCellEditor;
import org.eclipse.jface.viewers.FocusCellOwnerDrawHighlighter;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TableViewerEditor;
import org.eclipse.jface.viewers.TableViewerFocusCellManager;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.RGB;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.ViewPart;

/**
 * This is the main class for providing the overlay view.
 * 
 * @author Matthew Harward
 * @version 23/8/2009
 */
public class ActiveMappingView extends ViewPart {

	// Defined Strings
	private static final String DESCRIPTEXT = "Metric overlays for ";
	private static final String NOFILETEXT = "The is no open active and valid editor.";
	private static final String MET_PREF_URL = "metricsOverlay.preferences.MetricsPreferencePage";
	private static final String AUG_PREF_URL = "metricsOverlay.preferences.AugmentationPreferencePage";
	private static final String MAP_PREF_URL = "metricsOverlay.preferences.MappingPreferencePage";

	// Major Fields
	private Table table;
	private TableViewer tableViewer;

	// Create a ExampleTaskList and assign it to an instance variable
	private ActiveMappingList taskList = new ActiveMappingList(null);

	// Set the table column property names
	private final String ACTIVE_COLUMN = "active";
	private final String MAPPING_COLUMN = "mapping";
	private final String METRIC_COLUMN = "metric";
	private final String AUGMENTATION_COLUMN = "display";
	private final String COUNT_COLUMN = "count";
	private final String REMOVE_COLUMN = "remove";
	
	//Actual col names
	private final String COL_ACTIVE = "!";
	private final String COL_MAPPING = "Mapping";
	private final String COL_METRIC = "Metric";
	private final String COL_AUGMENT = "Augmentation";
	private final String COL_COUNT = "#";
	private final String COL_REMOVE = "X";

	// Set column names
	private String[] columnNames = new String[] { ACTIVE_COLUMN,
			MAPPING_COLUMN, METRIC_COLUMN, AUGMENTATION_COLUMN, COUNT_COLUMN,
			REMOVE_COLUMN };

	// Workaround for edit to select and update edit cell labels.
	protected String mapSelectedValue = "Edit Mapping:";
	protected String metSelectedValue = "Edit Metric:";
	protected String augSelectedValue = "Edit Augmentation:";

	//Edit background color
	private static final RGB EDIT_BACKGROUND_RGB = new RGB(200, 250, 170);

	/**
	 * Default constructor
	 */
	public ActiveMappingView() {
		this.setContentDescription(NOFILETEXT);
	}

	/**
	 * Default initialisation
	 */
	public void createPartControl(Composite parent) {
		// Create the table
		createTable(parent);

		// Create and setup the TableViewer
		createTableViewer();
		tableViewer.setContentProvider(new MappingContentProvider());
		tableViewer.setLabelProvider(new ActiveMappingLabelProvider());
		// The input for the table viewer is the instance of ExampleTaskList
		taskList = new ActiveMappingList(null);
		tableViewer.setInput(taskList);

		// /getSite().getWorkbenchWindow().getSelectionService().addSelectionListener(listener);
		MetricFileHandler.getInstance().getViewerUpdateAction()
				.addPropertyChangeListener(mfListener);

		// Currently stubs
		initializeToolBar();
		initializeMenu();
	}

	/**
	 * Listens to updates to the metric file and updates the view
	 */
	private IPropertyChangeListener mfListener = new IPropertyChangeListener() {

		@Override
		public void propertyChange(PropertyChangeEvent event) {
			MetricFile mf = (MetricFile) event.getNewValue();
			updateView(mf);
		}
	};

	/**
	 * Main update function.
	 * 
	 * @param mf
	 *            the current metric file.
	 */
	private void updateView(final MetricFile mf) {
		getControl().getDisplay().asyncExec(new Runnable() {

			@Override
			public void run() {
				MetricDocument mDoc = mf.getMetricDocument();
				int active = mDoc.numActiveMappings();
				int declared = mDoc.numDeclaredMappings();
				int total = mDoc.numMappings();
				// Use system default date formating
				DateFormat df = SimpleDateFormat.getDateTimeInstance();
				String time = df.format(mDoc.getTimeStamp());
				// Set the description string
				setContentDescription(DESCRIPTEXT + mf.getPath().toString()
						+ " [Visualisations: " + active + " active, " + declared
						+ " declared, " + total + " avaliable] (Last updated: " + time
						+ ")");
				taskList = new ActiveMappingList(mDoc);
				tableViewer.setInput(taskList);
			}
			
		});
	}

	/**
	 * Create the TableViewer
	 */
	private void createTableViewer() {

		tableViewer = new TableViewer(table);
		tableViewer.setUseHashlookup(true);

		tableViewer.setColumnProperties(columnNames);

		// Set the cell modifier for the viewer
		tableViewer.setCellModifier(new ActiveMappingCellModifier(this));
		// Set the default sorter for the viewer
		tableViewer.setSorter(new ActiveMappingSorter(
				ActiveMappingSorter.MAPPING, ActiveMappingSorter.DESC));
		// Set up the editing manager
		TableViewerFocusCellManager tvfcm = new TableViewerFocusCellManager(
				tableViewer, new FocusCellOwnerDrawHighlighter(tableViewer));
		ColumnViewerEditorActivationStrategy cveas = new ColumnViewerEditorActivationStrategy(
				tableViewer);
		TableViewerEditor.create(tableViewer, tvfcm, cveas,
				ColumnViewerEditor.DEFAULT);
		// ^^ Horrible huh?

		// Create the cell editors
		final CellEditor[] editors = new CellEditor[columnNames.length];

		// Column 1 : Active (Checkbox)
		editors[0] = new CheckboxCellEditor(table, SWT.CHECK | SWT.READ_ONLY);

		// Column 2 : Mapping (Dialog)
		final DialogCellEditor mappingDiag = new MappingColEditor(table, SWT.READ_ONLY);
		editors[1] = mappingDiag;

		// Column 3 : Metric (Dialog)
		final DialogCellEditor metricDiag = new MetricColEditor(table, SWT.READ_ONLY);
		editors[2] = metricDiag;

		// Column 4 : Augmentation (Dialog)
		final DialogCellEditor augDiag = new AugmentColEditor(table, SWT.READ_ONLY);
		editors[3] = augDiag;
		
		//Column 5 : Count - No editor
		editors[4] = null;
		
		// Column 6 : Remove (Dialog)
		DialogCellEditor rem = new RemoveColEditor(table, SWT.READ_ONLY);
		editors[5] = rem;

		// Assign the cell editors to the viewer
		tableViewer.setCellEditors(editors);

		tableViewer.getColumnViewerEditor().addEditorActivationListener(
				new EditorActivationListener(metricDiag, mappingDiag, augDiag));
	}

	/**
	 * Create the Table
	 */
	private void createTable(Composite parent) {
		int style = SWT.SINGLE | SWT.BORDER | SWT.H_SCROLL | SWT.V_SCROLL
				| SWT.FULL_SELECTION | SWT.HIDE_SELECTION;

		int NUMBER_COLUMNS = 6;

		table = new Table(parent, style);

		GridData gridData = new GridData(GridData.FILL_BOTH);
		gridData.grabExcessVerticalSpace = true;
		gridData.horizontalAlignment = GridData.FILL;
		gridData.horizontalSpan = NUMBER_COLUMNS - 1;
		table.setLayoutData(gridData);

		table.setLinesVisible(true);
		table.setHeaderVisible(true);

		// 1st column with image/checkboxes - NOTE: The SWT.CENTER has no
		// effect!!
		TableColumn column = new TableColumn(table, SWT.CENTER, 0);
		column.setText(COL_ACTIVE);
		column.setResizable(false);
		column.setToolTipText("Toggle Active");
		column.setMoveable(true);
		column.setWidth(22);
		column.addSelectionListener(sortingColListener(ActiveMappingSorter.ACTIVE));

		// 2nd column with mapping name
		column = new TableColumn(table, SWT.LEFT, 1);
		column.setText(COL_MAPPING);
		column.setMoveable(true);
		column.setWidth(175);
		column.addSelectionListener(sortingColListener(ActiveMappingSorter.MAPPING));

		// 3rd column with metric name
		column = new TableColumn(table, SWT.LEFT, 2);
		column.setText(COL_METRIC);
		column.setMoveable(true);
		column.setWidth(175);
		column.addSelectionListener(sortingColListener(ActiveMappingSorter.METRIC));

		// 4th column with augmentation name
		column = new TableColumn(table, SWT.LEFT, 3);
		column.setText(COL_AUGMENT);
		column.setMoveable(true);
		column.setWidth(175);
		column.addSelectionListener(sortingColListener(ActiveMappingSorter.AUGMENT));

		// Remove
		column = new TableColumn(table, SWT.CENTER, 4);
		column.setText(COL_COUNT);
		column.setMoveable(true);
		column.setToolTipText("Number of occurances");
		column.setWidth(50);
		column.addSelectionListener(sortingColListener(ActiveMappingSorter.COUNT));
		
		// Remove
		column = new TableColumn(table, SWT.CENTER, 5);
		column.setText(COL_REMOVE);
		column.setResizable(false);
		column.setMoveable(true);
		column.setToolTipText("Remove");
		column.setWidth(22);
	}

	/**
	 * @return
	 */
	private SelectionAdapter sortingColListener(final int columnToSort) {
		return new SelectionAdapter() {

			private boolean mapDesc;

			public void widgetSelected(SelectionEvent e) {
				boolean val;
				if(mapDesc){
					val = ActiveMappingSorter.DESC;
				} else
					val = ActiveMappingSorter.ASC;
				tableViewer.setSorter(new ActiveMappingSorter(
						columnToSort,val));
				mapDesc = !mapDesc;
			}
		};
	}

	private void initializeToolBar() {
		/*ToolBarManager toolbarManager = (ToolBarManager) getViewSite()
				.getActionBars().getToolBarManager();*/
	}

	private void initializeMenu() {
		/*IMenuManager menuManager = getViewSite().getActionBars()
				.getMenuManager();*/
	}

	/**
	 * @param cell
	 * @return
	 */
	private Label createEditBox(Composite cell, String text) {
		Label a = new Label(cell, SWT.READ_ONLY);
		a.setText(text);
		a.setBackground(new Color(null, EDIT_BACKGROUND_RGB));
		return a;
	}

	/**
	 * @param parent
	 * @return
	 */
	private Button getEditButton(Composite parent) {
		Button a = new Button(parent, SWT.PUSH);
		a.setImage(new ActiveMappingLabelProvider().getEditImage());
		a.setBackground(new Color(null, EDIT_BACKGROUND_RGB));
		a.setText("Edit");
		a.setToolTipText("Click to Edit");
		return a;
	}

	/**
	 * Return the ExampleTaskList
	 */
	public ActiveMappingList getTaskList() {
		return taskList;
	}

	/**
	 * @return currently selected item
	 */
	public ISelection getSelection() {
		return tableViewer.getSelection();
	}

	/**
	 * Return the column names in a collection
	 * 
	 * @return List containing column names
	 */
	public List<String> getColumnNames() {
		return Arrays.asList(columnNames);
	}

	/**
	 * Return the parent composite
	 */
	public Control getControl() {
		return table.getParent();
	}

	public void setFocus() {
		//table.setFocus();
	}

	public void dispose() {
		table.dispose();
		super.dispose();
	}

	private final class EditorActivationListener extends
			ColumnViewerEditorActivationListener {
		private final DialogCellEditor metricDiag;
		private final DialogCellEditor mappingDiag;
		private final DialogCellEditor augDiag;

		private EditorActivationListener(DialogCellEditor metricDiag,
				DialogCellEditor mappingDiag, DialogCellEditor augDiag) {
			this.metricDiag = metricDiag;
			this.mappingDiag = mappingDiag;
			this.augDiag = augDiag;
		}

		@Override
		public void beforeEditorDeactivated(
				ColumnViewerEditorDeactivationEvent event) {
		}

		@Override
		public void beforeEditorActivated(
				ColumnViewerEditorActivationEvent event) {

			Mapping m = (Mapping) tableViewer
					.getColumnViewerEditor().getFocusCell()
					.getElement();
			mapSelectedValue = m.getName();
			mappingDiag.activate();
			metSelectedValue = m.getMetric().getName();
			metricDiag.activate();
			augSelectedValue = m.getAugmentation().getName();
			augDiag.activate();

		}

		@Override
		public void afterEditorDeactivated(
				ColumnViewerEditorDeactivationEvent event) {
			mappingDiag.deactivate();
			metricDiag.deactivate();
			augDiag.deactivate();
		}

		@Override
		public void afterEditorActivated(
				ColumnViewerEditorActivationEvent event) {
		}
	}

	private final class MappingColEditor extends DialogCellEditor {
		private Label contentLab;

		private MappingColEditor(Composite parent, int style) {
			super(parent, style);
		}

		@Override
		public void activate() {
			if (mapSelectedValue.length() > 0)
				contentLab.setText(mapSelectedValue);
		}

		@Override
		protected Control createContents(Composite cell) {
			contentLab = createEditBox(cell, "Edit Mapping:");
			return contentLab;
		}

		@Override
		protected Button createButton(Composite parent) {
			return getEditButton(parent);
		}

		@Override
		protected Object openDialogBox(Control cellEditorWindow) {
			PreferenceDialog diag = org.eclipse.ui.dialogs.PreferencesUtil
					.createPreferenceDialogOn(PlatformUI.getWorkbench()
							.getActiveWorkbenchWindow().getShell(),
							MAP_PREF_URL, null, null);
			// Select appropriate mapping...
			MappingPreferencePage map = (MappingPreferencePage) diag
					.getSelectedPage();
			Mapping m = (Mapping) tableViewer.getColumnViewerEditor()
					.getFocusCell().getElement();
			map.getController().setActiveMapping(m);
			// Open the page
			diag.open();
			return null;
		}
	}

	private final class MetricColEditor extends DialogCellEditor {
		private Label contentLab;

		private MetricColEditor(Composite parent, int style) {
			super(parent, style);
		}

		@Override
		public void activate() {
			if (metSelectedValue.length() > 0)
				contentLab.setText(metSelectedValue);
		}

		@Override
		protected Control createContents(Composite cell) {
			contentLab = createEditBox(cell, metSelectedValue);
			return contentLab;
		}

		@Override
		protected Button createButton(Composite parent) {
			return getEditButton(parent);
		}

		@Override
		protected Object openDialogBox(Control cellEditorWindow) {
			PreferenceDialog diag = org.eclipse.ui.dialogs.PreferencesUtil
			.createPreferenceDialogOn(PlatformUI.getWorkbench()
					.getActiveWorkbenchWindow().getShell(),
					MET_PREF_URL, null, null);
			// Select appropriate metric...
			MetricsPreferencePage met = (MetricsPreferencePage) diag
			.getSelectedPage();
			Mapping m = (Mapping) tableViewer.getColumnViewerEditor()
			.getFocusCell().getElement();
			met.getController().setActiveMetric(m.getMetric());
			// Open the page
			diag.open();
			return null;
		}
	}

	private final class RemoveColEditor extends DialogCellEditor {
		private RemoveColEditor(Composite parent, int style) {
			super(parent, style);
		}

		@Override
		public void activate() {
			Mapping m = (Mapping) tableViewer.getColumnViewerEditor()
					.getFocusCell().getElement();
			Shell shell = table.getShell();
			MessageBox messageBox2 = new MessageBox(shell,
					SWT.ICON_QUESTION | SWT.YES | SWT.NO);
			messageBox2.setText("Remove Mapping");
			messageBox2.setMessage("Would you like to remove the \""
					+ m.getName() + "\" mapping?");
			if (messageBox2.open() == SWT.YES) {
				// Remove selected mapping
				tableViewer.remove(m);
				m.setMappingState(MappingState.UndeclaredMapping);
				MetricFileHandler.getInstance().forceUpdate();
			}
		}

		@Override
		protected Button createButton(Composite parent) {
			Button b = super.createButton(parent);
			b.setVisible(false);
			return b;
		}

		@Override
		protected Object openDialogBox(Control cellEditorWindow) {
			return null;
		}
	}

	private final class AugmentColEditor extends DialogCellEditor {
		private Label contentLab;

		private AugmentColEditor(Composite parent, int style) {
			super(parent, style);
		}

		@Override
		public void activate() {
			if (augSelectedValue.length() > 0)
				contentLab.setText(augSelectedValue);
		}

		@Override
		protected Control createContents(Composite cell) {
			contentLab = createEditBox(cell, augSelectedValue);
			return contentLab;
		}

		@Override
		protected Button createButton(Composite parent) {
			return getEditButton(parent);
		}

		@Override
		protected Object openDialogBox(Control cellEditorWindow) {
			// Set up pref page
			PreferenceDialog diag = org.eclipse.ui.dialogs.PreferencesUtil
					.createPreferenceDialogOn(PlatformUI.getWorkbench()
							.getActiveWorkbenchWindow().getShell(),
							AUG_PREF_URL, null, null);
			// diag.setBlockOnOpen(true);
			// Select appropriate augmentation...
			AugmentationPreferencePage aug = (AugmentationPreferencePage) diag
					.getSelectedPage();
			Mapping m = (Mapping) tableViewer.getColumnViewerEditor()
					.getFocusCell().getElement();

			aug.getController().setActiveAugmentation(m.getAugmentation());
			// Open the page
			diag.open();
			return null;
		}
	}

	/**
	 * InnerClass that acts as a proxy for the ExampleTaskList providing content
	 * for the Table. It implements the ITaskListViewer interface since it must
	 * register changeListeners with the ExampleTaskList
	 */
	class MappingContentProvider implements IStructuredContentProvider,
			IActiveMappingListViewer {
		public void inputChanged(Viewer v, Object oldInput, Object newInput) {
			if (newInput != null)
				((ActiveMappingList) newInput).addChangeListener(this);
			if (oldInput != null)
				((ActiveMappingList) oldInput).removeChangeListener(this);
		}

		public void dispose() {
			taskList.removeChangeListener(this);
		}

		// Return the tasks as an array of Objects
		public Object[] getElements(Object parent) {
			return taskList.getMappingList().toArray();
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see ITaskListViewer#addTask(ExampleTask)
		 */
		public void addMapping(Mapping mapping) {
			tableViewer.add(mapping);
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see ITaskListViewer#removeTask(ExampleTask)
		 */
		public void removeMapping(Mapping mapping) {
			tableViewer.remove(mapping);
		}

		@Override
		public void updateMapping(Mapping mapping) {
			tableViewer.update(mapping, null);

		}
	}

	public void updateModel(Mapping map) {
		MetricFileHandler.getInstance().updateMapping(map);
	}
}