package metricsOverlay.xml;

//Imports
import java.io.File;
import java.io.FileNotFoundException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Result;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.TransformerFactoryConfigurationError;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import metricsOverlay.model.MetricDocument;
import metricsOverlay.model.MetricSection;
import metricsOverlay.model.augmentation.Augmentation;
import metricsOverlay.model.augmentation.BackgroundLineAugmentation;
import metricsOverlay.model.augmentation.BackgroundSectionAugmentation;
import metricsOverlay.model.augmentation.MarginAugmentation;
import metricsOverlay.model.augmentation.UnderlineAugmentation;
import metricsOverlay.model.augmentation.other.ShapeDirection;
import metricsOverlay.model.augmentation.other.ShapeType;
import metricsOverlay.model.augmentation.other.UnderlineType;
import metricsOverlay.model.mappings.AugmentExponentialIStrategy;
import metricsOverlay.model.mappings.AugmentLogIStrategy;
import metricsOverlay.model.mappings.InterpolationStrategy;
import metricsOverlay.model.mappings.LinearIStrategy;
import metricsOverlay.model.mappings.Mapping;
import metricsOverlay.model.mappings.MappingState;
import metricsOverlay.model.mappings.MetricExponentialIStrategy;
import metricsOverlay.model.mappings.MetricLogIStrategy;
import metricsOverlay.model.metrics.ActualMetric;
import metricsOverlay.model.metrics.CompositeIntegerRange;
import metricsOverlay.model.metrics.CompositeRealRange;
import metricsOverlay.model.metrics.DerivedMetric;
import metricsOverlay.model.metrics.IntegerRange;
import metricsOverlay.model.metrics.MappedRange;
import metricsOverlay.model.metrics.Metric;
import metricsOverlay.model.metrics.OrderedNominalRange;
import metricsOverlay.model.metrics.Range;
import metricsOverlay.model.metrics.RealRange;

import org.eclipse.swt.graphics.Color;
import org.w3c.dom.Document;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.ErrorHandler;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;

/**
 * Main class for reading in XML files. The file "metric.dtd" describes the
 * appropriate XML format.
 * 
 * @author Matthew Harward
 * @version 3/9/2009
 */
public class MetricXMLReader {

	// Declarations
	private String fileName;
	private MetricDocument mDoc;
	private Map<Metric, String> metricMap;
	private long timeStamp;

	// Debug Flag
	private static final boolean DEBUG = false;

	/**
	 * Main constructor.
	 * 
	 * @param fileName
	 *            A string to store the resultant file name.
	 */
	public MetricXMLReader(String fileName) {
		this.fileName = fileName;
		mDoc = new MetricDocument(fileName);
		metricMap = new HashMap<Metric, String>();
		timeStamp = -1;
	}

	/**
	 * Constructor given an existing MetricDocument for the file.
	 * 
	 * @param mDoc
	 *            a document to contain all of the mappings etc
	 */
	@Deprecated
	public MetricXMLReader(MetricDocument mDoc) {
		this.mDoc = mDoc;
		fileName = mDoc.getFileName().toOSString();
		metricMap = new HashMap<Metric, String>();
		timeStamp = -1;
	}

	/**
	 * Reads a specific xmlFile
	 * 
	 * @param xmlFileName
	 */
	public void readXML(String xmlFileName) {
		try {
			File xmlFile = new File(xmlFileName);
			// TODO get relative file paths working...
			if (!xmlFile.exists())
				throw new FileNotFoundException("Doesn't exist");
			if (!xmlFile.canRead())
				throw new FileNotFoundException(
				"The provided path to the xml file could not be read");
			DocumentBuilderFactory factory = DocumentBuilderFactory
			.newInstance();
			factory.setValidating(true);
			DocumentBuilder builder = factory.newDocumentBuilder();
			ErrorHandler handle = new MyErrorHandler();
			builder.setErrorHandler(handle);
			Document doc = builder.parse(xmlFile);
			if (DEBUG)
				printXML(doc);
			readDom(doc);
		} catch (Exception e) {
			System.err.println(e.getMessage());
			e.printStackTrace();
		}
	}

	/**
	 * Creates a print out of the XML to System.out
	 * 
	 * @param doc
	 *            the XML DOM representation
	 * @throws TransformerConfigurationException
	 * @throws TransformerFactoryConfigurationError
	 * @throws TransformerException
	 */
	private void printXML(Document doc)
	throws TransformerConfigurationException,
	TransformerFactoryConfigurationError, TransformerException {
		// Create transformer
		Transformer tFormer = TransformerFactory.newInstance().newTransformer();
		// Output text type
		tFormer.setOutputProperty(OutputKeys.METHOD, XMLConstants.XML);
		// Write the document to a file
		Source source = new DOMSource(doc);
		Result result = new StreamResult(System.out);
		tFormer.transform(source, result);
	}

	// TODO make sure that errors are handled properly
	// This means that the error handler and any other error reporting is
	// controlling conjointly.
	private static class MyErrorHandler implements ErrorHandler {
		public void warning(SAXParseException e) throws SAXException {
			System.out.println("Warning: ");
			printInfo(e);
		}

		public void error(SAXParseException e) throws SAXException {
			System.out.println("Error: ");
			printInfo(e);
		}

		public void fatalError(SAXParseException e) throws SAXException {
			System.out.println("Fatal error: ");
			printInfo(e);
		}

		private void printInfo(SAXParseException e) {
			System.out.println("   Public ID: " + e.getPublicId());
			System.out.println("   System ID: " + e.getSystemId());
			System.out.println("   Line number: " + e.getLineNumber());
			System.out.println("   Column number: " + e.getColumnNumber());
			System.out.println("   Message: " + e.getMessage());
		}
	}

	/**
	 * Extracts all data contained in the XML.
	 * 
	 * @param doc
	 *            the XML DOM representation
	 */
	private void readDom(Document doc) {
		readFileName(doc);
		extractMetrics(doc);
		rationaliseMetrics();
		extractAugmentations(doc);
		//Do mappings last to allow names to be used to retrieve mappings.
		extractMappings(doc);
		//Add the timestamp if necessary...
		if(timeStamp > -1)
			mDoc.setTimeStamp(timeStamp);
	}

	/**
	 * Extracts the file name of the appropriate java file from the XML.
	 * 
	 * @param doc
	 *            the XML DOM representation
	 */
	private void readFileName(Document doc) {
		timeStamp = -1;
		NodeList rootList = doc.getElementsByTagName(XMLConstants.DOCUMENT);
		Node rootNode = rootList.item(0);
		NamedNodeMap rootAtts = rootNode.getAttributes();
		//Get filename
		Node fNameNode = rootAtts.getNamedItem(XMLConstants.FILENAME);
		String tempName = "";
		if (fNameNode != null)
			tempName = fNameNode.getNodeValue();
		if (tempName.length() > 0)
			fileName = tempName;
		//Get timestamp
		Node timeStampNode = rootAtts.getNamedItem(XMLConstants.TIMESTAMP);
		String tempTime = "";
		if (timeStampNode != null)
			tempTime = timeStampNode.getNodeValue();
		if (tempTime.length() > 0)
			timeStamp = Long.parseLong(tempTime);
	}

	/**
	 * Extracts all metrics from the XML file.
	 * 
	 * @param doc
	 *            the XML DOM representation
	 */
	private void extractMetrics(Document doc) {
		NodeList metricsList = doc.getElementsByTagName(XMLConstants.METRIC);
		for (int c = 0; c < metricsList.getLength(); c++) {
			Node curMetric = metricsList.item(c);
			NodeList m = curMetric.getChildNodes();
			// Initialise metric variables
			String metricName = "";
			boolean editable = true;
			String derivedName = null;
			Set<MetricSection> mss = new HashSet<MetricSection>();
			Range curRange = null;
			//Extract attributes
			NamedNodeMap metAtts = curMetric.getAttributes();
			Node nameNode = metAtts.getNamedItem(XMLConstants.NAME);
			if(nameNode != null)
				metricName = nameNode.getNodeValue();
			Node nEditable = metAtts.getNamedItem(XMLConstants.EDITABLE);
			if (nEditable != null){
				editable = Boolean.parseBoolean(nEditable.getNodeValue());
			}
			//Extract elements
			for (int i = 0; i < m.getLength(); i++) {
				// Extract properties of the metric
				Node cur = m.item(i);
				if (cur.getNodeName().equals(XMLConstants.SIMPLERANGE) || 
						cur.getNodeName().equals(XMLConstants.COMPOSITERANGE))
					curRange = extractRange(cur);
				else if (cur.getNodeName().equals(XMLConstants.METRICSECTION))
					mss.add(extractMetricSections(cur));
				else if (cur.getNodeName().equals(XMLConstants.DERIVEDMETRIC)){
					NamedNodeMap derMetricAtts = cur.getAttributes();
					Node fromNode = derMetricAtts.getNamedItem(XMLConstants.FROM);
					derivedName = fromNode.getNodeValue();
				}
			}
			//The the metric is valid it can be created.
			if (metricName.length() > 0 && curRange != null) {
				//If a derived name is specified any metric sections will be ignored.
				if(derivedName == null){
					ActualMetric curM = new ActualMetric(metricName, curRange);
					if(mss != null){
						curM.setMetricSections(mss);
					}
					metricMap.put(curM, "");
				} else {
					DerivedMetric derM = new DerivedMetric(metricName, curRange, null);
					derM.setEditable(editable);
					metricMap.put(derM, derivedName);
				}
			}
		}
	}

	/**
	 * Extracts all sections represented by a particular mapping from the XML.
	 * 
	 * @param cur
	 *            the Node holding the metricsection
	 */
	private MetricSection extractMetricSections(Node cur) {
		MetricSection finalMS = null;
		// Set up variables
		String sline = "";
		String schar = "";
		String eline = "";
		String echar = "";
		String value = "";
		// Extract values
		NamedNodeMap mSectionAtt = cur.getAttributes();
		if (mSectionAtt != null) {
			Node nsline = mSectionAtt.getNamedItem(XMLConstants.STARTLINE);
			if (nsline != null)
				sline = nsline.getNodeValue();
			Node nschar = mSectionAtt.getNamedItem(XMLConstants.STARTCOL);
			if (nschar != null)
				schar = nschar.getNodeValue();
			Node neline = mSectionAtt.getNamedItem(XMLConstants.ENDLINE);
			if (neline != null)
				eline = neline.getNodeValue();
			Node nechar = mSectionAtt.getNamedItem(XMLConstants.ENDCOL);
			if (nechar != null)
				echar = nechar.getNodeValue();
			Node nvalue = mSectionAtt.getNamedItem(XMLConstants.VALUE);
			if (nvalue != null)
				value = nvalue.getNodeValue();
			//Get actual text out
			String actualT = null;
			String annotate = null;
			NodeList sectionChildren = cur.getChildNodes();
			for(int g = 0; g < sectionChildren.getLength(); g++){
				if(sectionChildren.item(g).getNodeName().equals(XMLConstants.ACTUALTEXT)){
					actualT = sectionChildren.item(g).getTextContent();
				}
				if(sectionChildren.item(g).getNodeName().equals(XMLConstants.ANNOTATION)){
					annotate = sectionChildren.item(g).getTextContent();
				}
			}
			// Create metric section
			try {
				MetricSection ms = new MetricSection(Integer
						.parseInt(sline), Integer.parseInt(schar), Integer
						.parseInt(eline), Integer.parseInt(echar), value);
				if(actualT != null)
					ms.setActualText(actualT);
				if(annotate != null)
					ms.setAnnotation(annotate);
				finalMS = ms;
			} catch (NumberFormatException e) {
			} catch (Exception e) {
			}
		}
		return finalMS;
	}

	private void rationaliseMetrics() {
		for(Metric met : metricMap.keySet()) {
			if(met instanceof DerivedMetric){
				String dName = metricMap.get(met);
				for(Metric met2 : metricMap.keySet()){
					if(met2 instanceof ActualMetric && met2.getName().equals(dName)){
						((DerivedMetric) met).setUnderlyingMetric((ActualMetric) met2);
					}
				}

			}
			//Can be empty, not null.
			if(met.getMetricSections() != null)
				mDoc.addMetric(met);
		}
	}

	/**
	 * Extracts the range of a metric from the XML.
	 * 
	 * @param cur
	 *            the node the range resides in
	 * @return the range object
	 */
	private Range extractRange(Node cur) {
		Range finalRange = null;
		if(cur.getNodeName().equals(XMLConstants.SIMPLERANGE))
			finalRange = extractSimpleRange(cur);
		else if(cur.getNodeName().equals(XMLConstants.COMPOSITERANGE))
			finalRange = extactCompositeRange(cur);
		return finalRange;
	}
	
	private Range extractSimpleRange(Node cur) {
		NamedNodeMap curAtts = cur.getAttributes();
		//Intialise variables.
		String type = "";
		String max = "";
		String min = "";
		//Extract data
		Node nType = curAtts.getNamedItem(XMLConstants.TYPE);
		if(nType != null){
			type = nType.getNodeValue();
		}
		Node nMin = curAtts.getNamedItem(XMLConstants.MIN);
		if(nMin != null){
			min = nMin.getNodeValue();
		}
		Node nMax = curAtts.getNamedItem(XMLConstants.MAX);
		if(nMax != null){
			max = nMax.getNodeValue();
		}
		//Set up range
		Range ran = null;
		if(type.equals(XMLConstants.INTEGER)){
			ran = new IntegerRange(Integer.parseInt(min),
					Integer.parseInt(max));
		} else if(type.equals(XMLConstants.REAL)){
			ran = new RealRange(Double.parseDouble(min),
					Double.parseDouble(max));
		}
		return ran;
	}

	private Range extactCompositeRange(Node cur) {
		MappedRange fRange = null;
		NamedNodeMap curAtts = cur.getAttributes();
		NodeList compositeParts = cur.getChildNodes();
		if(curAtts != null && compositeParts != null){
			String type = "";
			Node nType = curAtts.getNamedItem(XMLConstants.TYPE);
			if(nType != null)
				type = nType.getNodeValue();
			if(type.length() > 0){
				if(type.equals(XMLConstants.ORDEREDNOMINAL)){
					Map<Integer,String> on = getOrderedNominals(compositeParts);
					fRange = new OrderedNominalRange(on);
				} else if (type.equals(XMLConstants.COMPOSITEINT)){
					Map<IntegerRange,String> compis = getCompositeInts(compositeParts);
					fRange = new CompositeIntegerRange(compis);
				} else if (type.equals(XMLConstants.COMPOSITEREAL)){
					Map<RealRange,String> comprs = getCompositeReals(compositeParts);
					fRange = new CompositeRealRange(comprs);
				}
			}
		}
		return fRange;
	}

	

	private Map<RealRange, String> getCompositeReals(NodeList compositeParts) {
		Map<RealRange, String> realRange = new HashMap<RealRange, String>();
		for(int i = 0; i < compositeParts.getLength(); i++){
			Node curNode = compositeParts.item(i);
			if(curNode.getNodeName().equals(XMLConstants.RANGEITEM)){
				//Set up variables
				String name = "";
				RealRange rr = null;
				//Extract name
				NamedNodeMap nnm = curNode.getAttributes();
				Node nname = nnm.getNamedItem(XMLConstants.NAME);
				if(nname != null)
					name = nname.getNodeValue();
				NodeList rangeItemParts = curNode.getChildNodes();
				for(int j = 0; j < rangeItemParts.getLength(); j++){
					Node curSimpleRange = rangeItemParts.item(j);
					if(curSimpleRange.getNodeName().equals(XMLConstants.REAL))
						rr = (RealRange) extractSimpleRange(curSimpleRange);
				}
				//If range/string pair is valid, add it to the map
				if(name.length() > 0 && rr != null){
					realRange.put(rr,name);
				}
			}
		}
		return realRange;
	}

	private Map<IntegerRange, String> getCompositeInts(NodeList compositeParts) {
		Map<IntegerRange, String> intRange = new HashMap<IntegerRange, String>();
		for(int i = 0; i < compositeParts.getLength(); i++){
			Node curNode = compositeParts.item(i);
			if(curNode.getNodeName().equals(XMLConstants.RANGEITEM)){
				//Set up variables
				String name = "";
				IntegerRange ir = null;
				//Extract name
				NamedNodeMap nnm = curNode.getAttributes();
				Node nname = nnm.getNamedItem(XMLConstants.NAME);
				if(nname != null)
					name = nname.getNodeValue();
				NodeList rangeItemParts = curNode.getChildNodes();
				for(int j = 0; j < rangeItemParts.getLength(); j++){
					Node curSimpleRange = rangeItemParts.item(j);
					if(curSimpleRange.getNodeName().equals(XMLConstants.INTEGER))
						ir = (IntegerRange) extractSimpleRange(curSimpleRange);
				}
				//If range/string pair is valid, add it to the map
				if(name.length() > 0 && ir != null){
					intRange.put(ir,name);
				}
			}
		}
		return intRange;
	}

	private Map<Integer, String> getOrderedNominals(NodeList compositeParts) {
		Map<Integer, String> onRange = new HashMap<Integer, String>();
		for(int i = 0; i < compositeParts.getLength(); i++){
			Node curNode = compositeParts.item(i);
			if(curNode.getNodeName().equals(XMLConstants.RANGEITEM)){
				//Set up variables
				String name = "";
				Integer inte = null;
				//Extract name
				NamedNodeMap nnm = curNode.getAttributes();
				Node nname = nnm.getNamedItem(XMLConstants.NAME);
				if(nname != null)
					name = nname.getNodeValue();
				NodeList rangeItemParts = curNode.getChildNodes();
				for(int j = 0; j < rangeItemParts.getLength(); j++){
					Node curSimpleRange = rangeItemParts.item(j);
					if(curSimpleRange.getNodeName().equals(XMLConstants.VALUE)){
						try{
						inte = Integer.parseInt(curSimpleRange.getTextContent());
						}catch(NumberFormatException e){}
					}	
				}
				//If range/string pair is valid, add it to the map
				if(name.length() > 0 && inte != null){
					onRange.put(inte,name);
				}
			}
		}
		return onRange;
	}

	private void extractAugmentations(Document doc) {
		NodeList augList = doc.getElementsByTagName(XMLConstants.AUGMENTATION);
		for (int i = 0; i < augList.getLength(); i++){
			Node curAug = augList.item(i);
			NamedNodeMap attList = curAug.getAttributes();
			if(attList != null){
				//Set Up Variables
				String name = "";
				String type = "";
				int importance = -1;
				boolean discrete = false;
				boolean annotation = false;
				UnderlineType uType = null;
				ShapeDirection shapeDir = null;
				ShapeType shapeType = null;
				boolean onLeft = false;
				boolean editable = true;

				//Extract Attributes
				Node naugName = attList.getNamedItem(XMLConstants.NAME);
				if (naugName != null)
					name = naugName.getNodeValue();	
				Node naugType = attList.getNamedItem(XMLConstants.TYPE);
				if (naugType != null)
					type = naugType.getNodeValue();	
				Node nimport = attList.getNamedItem(XMLConstants.IMPORTANCE);
				if (nimport != null){
					try {
						importance = Integer.valueOf(nimport.getNodeValue());
					} catch (NumberFormatException e) {}
				}
				Node ndiscrete = attList.getNamedItem(XMLConstants.DISCRETE);
				if (ndiscrete != null)
					discrete = Boolean.valueOf(ndiscrete.getNodeValue());
				Node nannotate = attList.getNamedItem(XMLConstants.ANNOTATION);
				if (nannotate != null){ 
					String anno = nannotate.getNodeValue();	
					if(anno.equals(XMLConstants.ON)) annotation = true;
				}
				Node nunder = attList.getNamedItem(XMLConstants.UNDERLINE);
				if (nunder != null){
					String uVal = nunder.getNodeValue();	
					if (uVal.equals(XMLConstants.SWIGGLE))
						uType = UnderlineType.SwiggleUnderline;
					else if (uVal.equals(XMLConstants.LINK))
						uType = UnderlineType.LinkUnderline;
					else if (uVal.equals(XMLConstants.DOUBLE))
						uType = UnderlineType.DoubleUnderline;
					else if (uVal.equals(XMLConstants.SINGLE))
						uType = UnderlineType.SingleUnderline;
				}
				Node nshapeType = attList.getNamedItem(XMLConstants.SHAPETYPE);
				if (nshapeType != null){
					String sType = nshapeType.getNodeValue();	
					//Made need error checking...
					shapeType = ShapeType.valueOf(sType);
				}
				Node nShapeDir = attList.getNamedItem(XMLConstants.SHAPEDIR);
				if (nShapeDir != null){
					String sDir = nShapeDir.getNodeValue();	
					//Made need error checking...
					shapeDir = ShapeDirection.valueOf(sDir);
				}
				Node nMarginSide = attList.getNamedItem(XMLConstants.MARGINSIDE);
				if (nMarginSide != null){
					if(nMarginSide.getNodeValue().equals(XMLConstants.LEFT)){
						onLeft = true;
					}
				}
				Node nEditable = attList.getNamedItem(XMLConstants.EDITABLE);
				if (nEditable != null){
					editable = Boolean.parseBoolean(nEditable.getNodeValue());
				}
				//Get colors
				List<Color> colors = extractColors(curAug);

				//Set up augmentation
				if(type.length() > 0 && name.length() > 0){
					Augmentation au = null;
					if(type.equals(XMLConstants.MARGINTYPE))
						au = new MarginAugmentation(name);
					else if(type.equals(XMLConstants.UNDERLINETYPE))
						au = new UnderlineAugmentation(name);
					else if(type.equals(XMLConstants.BGSECTIONTYPE))
						au = new BackgroundSectionAugmentation(name);
					else if(type.equals(XMLConstants.BGLINETYPE))
						au = new BackgroundLineAugmentation(name);
					if(au != null){
					au.setAnnotations(annotation);
					au.setImportance(importance);
					au.setDiscrete(discrete);
					au.setColors(colors);
					au.setEditable(editable);
					if(au instanceof MarginAugmentation){
						((MarginAugmentation)au).setIsLeft(onLeft);
						((MarginAugmentation)au).setShape(shapeType, shapeDir);
					} else if(au instanceof UnderlineAugmentation){
						((UnderlineAugmentation)au).setUnderlineType(uType);
					}
					//Add the augmentation to the metric document.
					mDoc.addAugmentation(au);
					}
				}


			}
		}

	}

	private List<Color> extractColors(Node curAug) {
		NodeList colNodes = curAug.getChildNodes();
		List<Color> colList = new ArrayList<Color>();
		for(int i = 0; i < colNodes.getLength(); i++){
			Node curNode = colNodes.item(i);
			if(curNode.getNodeName().equals(XMLConstants.COLOR)){
				int red = -1;
				int green = -1;
				int blue = -1;
				NamedNodeMap colAtts = curNode.getAttributes();
				try{
					Node nred = colAtts.getNamedItem(XMLConstants.RED);
					if (nred != null)
						red = Integer.parseInt(nred.getNodeValue());
					Node ngreen = colAtts.getNamedItem(XMLConstants.GREEN);
					if (ngreen != null)
						green = Integer.parseInt(ngreen.getNodeValue());
					Node nblue = colAtts.getNamedItem(XMLConstants.BLUE);
					if (nblue != null)
						blue = Integer.parseInt(nblue.getNodeValue());
					colList.add(new Color(null,red,green,blue));
				} catch (Exception e){}
			}
		}
		return colList;
	}

	/**
	 * Extracts all mappings from the XML.
	 * 
	 * @param doc
	 *            the XML DOM representation
	 */
	private void extractMappings(Document doc) {
		NodeList mappingList = doc.getElementsByTagName(XMLConstants.MAPPING);
		for (int i = 0; i < mappingList.getLength(); i++) {
			Node curMapping = mappingList.item(i);
			NamedNodeMap attList = curMapping.getAttributes();
			if (attList != null) {
				// Variables
				String mapName = "";
				String augName = "";
				String metricName = "";
				boolean editable = false;
				MappingState mstate = MappingState.UndeclaredMapping;
				String interMeth = "";
				String interConst = "";
				// Extract attributes
				Node nmapName = attList.getNamedItem(XMLConstants.MAPPINGNAME);
				if (nmapName != null)
					mapName = nmapName.getNodeValue();
				Node ndisplayName = attList
				.getNamedItem(XMLConstants.AUGMENTATIONNAME);
				if (ndisplayName != null)
					augName = ndisplayName.getNodeValue();
				Node nmetricName = attList
				.getNamedItem(XMLConstants.METRICNAME);
				if (nmetricName != null)
					metricName = nmetricName.getNodeValue();
				Node nedit = attList.getNamedItem(XMLConstants.EDITABLE);
				if(nedit != null)
					editable = Boolean.parseBoolean(nedit.getNodeName());
				Node nState = attList.getNamedItem(XMLConstants.STATE);
				if(nState != null){
					String state = nState.getNodeValue();
					if(state.equals(XMLConstants.DECLARED)){
						mstate = MappingState.DeclaredMapping;
					} else if (state.equals(XMLConstants.ACTIVE)){
						mstate = MappingState.ActiveMapping;
					}
				}
				Node nInterMethod = attList.getNamedItem(XMLConstants.INTERMETHOD);
				if(nInterMethod != null)
					interMeth = nInterMethod.getNodeValue();
				Node nInterConst = attList.getNamedItem(XMLConstants.INTERCONST);
				if(nInterConst != null)
					interConst = nInterConst.getNodeValue();
				// Create objects
				if(mapName.length() > 0 && augName.length() > 0 && metricName.length() > 0){
					Mapping curMap = new Mapping(mapName);
					curMap.setMetric(mDoc.getMetric(metricName));
					curMap.setAugmentation(mDoc.getAugmentation(augName));
					curMap.setEditable(editable);
					InterpolationStrategy iStrat = generateInterpolationStrat(
							interMeth, interConst);
					curMap.setInterpolationType(iStrat);
					curMap.setMappingState(mstate);
					mDoc.addMapping(curMap);
				}
			}
		}
	}

	/**
	 * @param interMeth
	 * @param interConst
	 * @return
	 */
	private InterpolationStrategy generateInterpolationStrat(String interMeth,
			String interConst) {
		InterpolationStrategy iStrat = null;
		if(interMeth.length() > 0){
			if(interMeth.equals(XMLConstants.LINEAR))
				iStrat = new LinearIStrategy();
			else if(interMeth.equals(XMLConstants.METEXP)){
				MetricExponentialIStrategy temp = 
					new MetricExponentialIStrategy();
				if(interConst.length() > 0 )
					temp.setConstant(Double.parseDouble(interConst));
				else temp.setConstant(1);
				iStrat = temp;
			}
			else if(interMeth.equals(XMLConstants.AUGEXP)){
				AugmentExponentialIStrategy temp = 
					new AugmentExponentialIStrategy();
				if(interConst.length() > 0 )
					temp.setConstant(Double.parseDouble(interConst));
				else temp.setConstant(1);
				iStrat = temp;
			}
			else if(interMeth.equals(XMLConstants.METLOG)){
				MetricLogIStrategy temp = 
					new MetricLogIStrategy();
				if(interConst.length() > 0 )
					temp.setConstant(Double.parseDouble(interConst));
				else temp.setConstant(1);
				iStrat = temp;
			}
			else if(interMeth.equals(XMLConstants.AUGLOG)){
				AugmentLogIStrategy temp = 
					new AugmentLogIStrategy();
				if(interConst.length() > 0 )
					temp.setConstant(Double.parseDouble(interConst));
				else temp.setConstant(1);
				iStrat = temp;
			}
		}
		//Default if no strat set
		if(iStrat == null)
			iStrat = new LinearIStrategy();
		return iStrat;
	}

	// Getters
	/**
	 * @return the fileName listed in the xml
	 */
	public String getFileName() {
		return fileName;
	}

	/**
	 * @return a document containing all of the mappings and the appropriate
	 *         locations
	 */
	public MetricDocument getMetricDocument() {
		return mDoc;
	}

	/**
	 * @return a map object containing the metrics listed in the XML file. The
	 *         initial string represents their name.
	 */
	public Map<Metric,String> getMetricMap() {
		return metricMap;
	}
}
