package metricsOverlay.xml;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import metricsOverlay.model.MetricDocument;
import metricsOverlay.model.MetricSection;
import metricsOverlay.model.augmentation.Augmentation;
import metricsOverlay.model.augmentation.BackgroundSectionAugmentation;
import metricsOverlay.model.augmentation.MarginAugmentation;
import metricsOverlay.model.augmentation.UnderlineAugmentation;
import metricsOverlay.model.augmentation.other.UnderlineType;
import metricsOverlay.model.mappings.AugmentExponentialIStrategy;
import metricsOverlay.model.mappings.AugmentLogIStrategy;
import metricsOverlay.model.mappings.InterpolationStrategy;
import metricsOverlay.model.mappings.LinearIStrategy;
import metricsOverlay.model.mappings.Mapping;
import metricsOverlay.model.mappings.MappingState;
import metricsOverlay.model.mappings.MetricExponentialIStrategy;
import metricsOverlay.model.mappings.MetricLogIStrategy;
import metricsOverlay.model.metrics.CompositeIntegerRange;
import metricsOverlay.model.metrics.CompositeRealRange;
import metricsOverlay.model.metrics.DerivedMetric;
import metricsOverlay.model.metrics.IntegerRange;
import metricsOverlay.model.metrics.MappedRange;
import metricsOverlay.model.metrics.Metric;
import metricsOverlay.model.metrics.OrderedNominalRange;
import metricsOverlay.model.metrics.Range;
import metricsOverlay.model.metrics.RealRange;

import org.eclipse.swt.graphics.Color;
import org.w3c.dom.DOMException;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

/**
 * This class creates and writes XML to the provided file location. The contents
 * of the file to be written are provided in the constructor in the form of a
 * MetricDocument data structure.
 * 
 * @author Matthew Harward
 * @version 3/9/2009
 */
public class MetricXMLWriter {

	// The metric document to be used
	private MetricDocument mDoc;

	/**
	 * The main constructor.
	 * 
	 * @param mDoc
	 *            a metric document containing a programatic representation of
	 *            the XML.
	 */
	public MetricXMLWriter(MetricDocument mDoc) {
		this.mDoc = mDoc;
	}

	/**
	 * This method constructs a XML file from the provided metrics document and
	 * saves it to the XML file location provided.
	 * 
	 * @param xmlFileName
	 *            the absolute path and name of the XML file to be saved. This
	 *            should be OS compatible and end with an .xml extension.
	 */
	public void writeFile(String xmlFileName) {
		try {
			// Make sure the file exists
			File f = new File(xmlFileName);
			f.createNewFile();

			// Construct a document to store the DOM
			DocumentBuilderFactory factory = DocumentBuilderFactory
					.newInstance();
			factory.setValidating(true);
			DocumentBuilder builder = factory.newDocumentBuilder();
			Document doc = builder.newDocument();
			doc.setXmlVersion("1.0");
			doc
					.appendChild(doc
							.createComment("XML file was serialised by MetricXMLWriter"));

			// Use the metrics document to create the contents of the DOM
			createDOMContents(doc);

			// Create an XSLT transform to write out the final XML
			DOMSource source = new DOMSource(doc);
			StreamResult result = new StreamResult(new FileOutputStream(
					xmlFileName));
			TransformerFactory transFactory = TransformerFactory.newInstance();
			Transformer transformer = transFactory.newTransformer();

			transformer.setOutputProperty(OutputKeys.INDENT, "yes");
			transformer.setOutputProperty(OutputKeys.DOCTYPE_SYSTEM, XMLManager
					.getAbsoluteDTDPath());

			transformer.transform(source, result);

		} catch (DOMException e) {
			e.printStackTrace();
		} catch (ParserConfigurationException e) {
			e.printStackTrace();
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (TransformerConfigurationException e) {
			e.printStackTrace();
		} catch (TransformerException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}

	}

	/**
	 * A method to create the contents of a DOM with the existing metrics
	 * document.
	 * 
	 * @param doc
	 *            the document representing the DOM
	 */
	private void createDOMContents(Document doc) {
		Element docEl = doc.createElement(XMLConstants.DOCUMENT);
		docEl.setAttribute(XMLConstants.FILENAME, mDoc.getFileName()
				.lastSegment());
		docEl.setAttribute(XMLConstants.TIMESTAMP, String.valueOf(mDoc
				.getTimeStamp().getTime()));

		List<Element> eaugments = extractAugmentations(doc, mDoc
				.getAllAugmentations());
		List<Element> emetrics = extractMetrics(doc, mDoc.getAllMetrics());
		List<Element> emappings = extractMappings(doc, mDoc.getAllMappings());

		// All all components to the root node
		docEl.appendChild(doc.createComment("All Metrics"));
		for (Element e1 : emetrics) {
			docEl.appendChild(e1);
		}
		docEl.appendChild(doc.createComment("All Mappings"));
		for (Element e2 : emappings) {
			docEl.appendChild(e2);
		}
		docEl.appendChild(doc.createComment("All Augmentations"));
		for (Element e3 : eaugments) {
			docEl.appendChild(e3);
		}
		doc.appendChild(docEl);
	}

	/**
	 * Creates a set of elements that provide a DOM representation of the
	 * augmentations in the MetricDocument.
	 */
	private List<Element> extractAugmentations(Document doc,
			Set<Augmentation> allAugmentations) {
		List<Element> augEls = new ArrayList<Element>();
		// Create a new element for each augmentation
		for (Augmentation aug : allAugmentations) {
			Element curAugEl = doc.createElement(XMLConstants.AUGMENTATION);
			curAugEl.setAttribute(XMLConstants.NAME, aug.getName());
			curAugEl.setAttribute(XMLConstants.IMPORTANCE, String.valueOf(aug
					.getImportance()));
			curAugEl.setAttribute(XMLConstants.DISCRETE, String.valueOf(aug
					.isDiscrete()));
			if (aug.isAnnotated())
				curAugEl.setAttribute(XMLConstants.ANNOTATION, XMLConstants.ON);
			else
				curAugEl
						.setAttribute(XMLConstants.ANNOTATION, XMLConstants.OFF);
			curAugEl.setAttribute(XMLConstants.EDITABLE, String.valueOf(aug
					.canEdit()));
			// Specific output for different types of augmentation
			if (aug instanceof MarginAugmentation) {
				curAugEl.setAttribute(XMLConstants.TYPE,
						XMLConstants.MARGINTYPE);
				MarginAugmentation ma = (MarginAugmentation) aug;
				curAugEl.setAttribute(XMLConstants.SHAPETYPE, ma.getShapeType()
						.name());
				// Enum may not change...
				curAugEl.setAttribute(XMLConstants.SHAPEDIR, ma
						.getShapeDirection().name());
				if (ma.isLeft())
					curAugEl.setAttribute(XMLConstants.MARGINSIDE,
							XMLConstants.LEFT);
				else
					curAugEl.setAttribute(XMLConstants.MARGINSIDE,
							XMLConstants.RIGHT);
			} else if (aug instanceof UnderlineAugmentation) {
				curAugEl.setAttribute(XMLConstants.TYPE,
						XMLConstants.UNDERLINETYPE);
				UnderlineAugmentation ua = (UnderlineAugmentation) aug;
				if (ua.getUnderlineType().equals(UnderlineType.DoubleUnderline)) {
					curAugEl.setAttribute(XMLConstants.UNDERLINE,
							XMLConstants.DOUBLE);
				} else if (ua.getUnderlineType().equals(
						UnderlineType.LinkUnderline)) {
					curAugEl.setAttribute(XMLConstants.UNDERLINE,
							XMLConstants.LINK);
				} else if (ua.getUnderlineType().equals(
						UnderlineType.SingleUnderline)) {
					curAugEl.setAttribute(XMLConstants.UNDERLINE,
							XMLConstants.SINGLE);
				} else
					curAugEl.setAttribute(XMLConstants.UNDERLINE,
							XMLConstants.SWIGGLE);
			} else if (aug instanceof BackgroundSectionAugmentation) {
				curAugEl.setAttribute(XMLConstants.TYPE,
						XMLConstants.BGSECTIONTYPE);
			} else
				curAugEl.setAttribute(XMLConstants.TYPE,
						XMLConstants.BGLINETYPE);
			// Add Colours
			List<Element> colorEls = new ArrayList<Element>();
			for (Color col : aug.getColors()) {
				Element curCEl = doc.createElement(XMLConstants.COLOR);
				curCEl.setAttribute(XMLConstants.RED, String.valueOf(col
						.getRed()));
				curCEl.setAttribute(XMLConstants.GREEN, String.valueOf(col
						.getGreen()));
				curCEl.setAttribute(XMLConstants.BLUE, String.valueOf(col
						.getBlue()));
				colorEls.add(curCEl);
			}
			for (Element e1 : colorEls) {
				curAugEl.appendChild(e1);
			}
			augEls.add(curAugEl);
		}
		return augEls;
	}

	/**
	 * Creates a set of elements that provide a DOM representation of the
	 * metrics in the MetricDocument.
	 */
	private List<Element> extractMetrics(Document doc, Set<Metric> allMetrics) {
		List<Element> emetrics = new ArrayList<Element>();
		for (Metric met : allMetrics) {
			Element metEl = doc.createElement(XMLConstants.METRIC);
			metEl.setAttribute(XMLConstants.NAME, met.getName());
			metEl.setAttribute(XMLConstants.EDITABLE, String.valueOf(met
					.canEdit()));
			if (met instanceof DerivedMetric) {
				Element derMetric = doc
						.createElement(XMLConstants.DERIVEDMETRIC);
				derMetric.setAttribute(XMLConstants.FROM, ((DerivedMetric) met)
						.derivedFrom());
				metEl.appendChild(derMetric);
			} else { // Actual Metric
				List<Element> metSections = extractMetricSections(doc, met);
				for (Element e1 : metSections) {
					metEl.appendChild(e1);
				}
			}
			// Sort out the range
			Range rr = met.getRange();
			if (rr instanceof MappedRange) {
				Element compRange = doc
						.createElement(XMLConstants.COMPOSITERANGE);
				String type = "";
				// Create an element from an ordered nominal range
				if (rr instanceof OrderedNominalRange) {
					type = XMLConstants.ORDEREDNOMINAL;
					Map<Integer, String> rm = ((OrderedNominalRange) rr)
							.getRangeMap();
					for (Integer integer : rm.keySet()) {
						Element valItem = doc
								.createElement(XMLConstants.RANGEITEM);
						valItem
								.setAttribute(XMLConstants.NAME, rm
										.get(integer));
						Element intItem = doc.createElement(XMLConstants.VALUE);
						intItem.setTextContent(String.valueOf(integer));
						valItem.appendChild(intItem);
						compRange.appendChild(valItem);
					}
					// Create an element from a composite integer range
				} else if (rr instanceof CompositeIntegerRange) {
					type = XMLConstants.COMPOSITEINT;
					Map<IntegerRange, String> rm = ((CompositeIntegerRange) rr)
							.getRangeMap();
					for (IntegerRange ir : rm.keySet()) {
						Element valItem = doc
								.createElement(XMLConstants.RANGEITEM);
						valItem.setAttribute(XMLConstants.NAME, rm.get(ir));
						Element intItem = extractSimpleRange(doc, ir);
						valItem.appendChild(intItem);
						compRange.appendChild(valItem);
					}
					// Create an element from a composite real range
				} else if (rr instanceof CompositeRealRange) {
					type = XMLConstants.COMPOSITEREAL;
					Map<RealRange, String> rm = ((CompositeRealRange) rr)
							.getRangeMap();
					for (RealRange rrr : rm.keySet()) {
						Element valItem = doc
								.createElement(XMLConstants.RANGEITEM);
						valItem.setAttribute(XMLConstants.NAME, rm.get(rrr));
						Element intItem = extractSimpleRange(doc, rrr);
						valItem.appendChild(intItem);
						compRange.appendChild(valItem);
					}
				}
				compRange.setAttribute(XMLConstants.TYPE, type);
				metEl.appendChild(compRange);
			} else {
				// Create a real/integer range element
				Element simpleRange = extractSimpleRange(doc, rr);
				metEl.appendChild(simpleRange);
			}
			// Add metric element to the list
			emetrics.add(metEl);
		}
		return emetrics;
	}

	/**
	 * Extracts an element that represents the provided range.
	 */
	private Element extractSimpleRange(Document doc, Range rr) {
		Element simpleRange = doc.createElement(XMLConstants.SIMPLERANGE);
		// If Integer Range
		if (rr instanceof IntegerRange)
			simpleRange.setAttribute(XMLConstants.TYPE, XMLConstants.INTEGER);
		// If Real Range
		else
			simpleRange.setAttribute(XMLConstants.TYPE, XMLConstants.REAL);
		// Set the min and max
		simpleRange.setAttribute(XMLConstants.MIN, rr.getMin());
		simpleRange.setAttribute(XMLConstants.MAX, rr.getMax());
		return simpleRange;
	}

	/**
	 * A method to extract metric sections that use the current metric into a
	 * list of elements that can be used by the DOM.
	 */
	private List<Element> extractMetricSections(Document doc, Metric curMet) {
		List<Element> metSections = new ArrayList<Element>();
		// Add all metric sections for this metric
		for (MetricSection mSec : curMet.getMetricSections()) {
			Element curSecEl = doc.createElement(XMLConstants.METRICSECTION);
			curSecEl.setAttribute(XMLConstants.STARTLINE, Integer.toString(mSec
					.getStartLine()));
			curSecEl.setAttribute(XMLConstants.STARTCOL, Integer.toString(mSec
					.getStartCol()));
			curSecEl.setAttribute(XMLConstants.ENDLINE, Integer.toString(mSec
					.getEndLine()));
			curSecEl.setAttribute(XMLConstants.ENDCOL, Integer.toString(mSec
					.getEndCol()));
			curSecEl.setAttribute(XMLConstants.VALUE, mSec.getValue());

			// If the metric section has actual text, add it
			if (mSec.hasActualText()) {
				Element curActualEl = doc
						.createElement(XMLConstants.ACTUALTEXT);
				curActualEl.setTextContent(mSec.getActualText());
				curSecEl.appendChild(curActualEl);
			}
			if (mSec.hasAnnotation()) {
				Element curAnnotateEl = doc
						.createElement(XMLConstants.ANNOTATION);
				curAnnotateEl.setTextContent(mSec.getAnnotation());
				curSecEl.appendChild(curAnnotateEl);
			}
			metSections.add(curSecEl);
		}
		return metSections;
	}

	/**
	 * A list of elements that represent the different mappings in the current
	 * MetricDocument. These can be added to the DOM.
	 */
	private List<Element> extractMappings(Document doc, Set<Mapping> mappings) {
		List<Element> emappings = new ArrayList<Element>();
		for (Mapping curMap : mappings) {
			if (!curMap.getMappingState().equals(MappingState.StubMapping)) {
				// Add the current mapping to the DOM
				Element curMapEl = doc.createElement(XMLConstants.MAPPING);
				curMapEl.setAttribute(XMLConstants.MAPPINGNAME, curMap
						.getName());
				curMapEl.setAttribute(XMLConstants.AUGMENTATIONNAME, curMap
						.getAugmentation().getName());
				curMapEl.setAttribute(XMLConstants.METRICNAME, curMap
						.getMetric().getName());
				// States
				String state = null;
				if (curMap.getMappingState().equals(MappingState.ActiveMapping))
					state = MappingState.ActiveMapping.name();
				else if (curMap.getMappingState().equals(
						MappingState.DeclaredMapping))
					state = MappingState.DeclaredMapping.name();
				else
					state = MappingState.UndeclaredMapping.name();
				curMapEl.setAttribute(XMLConstants.STATE, state);
				// Editable
				curMapEl.setAttribute(XMLConstants.EDITABLE, String
						.valueOf(curMap.canEdit()));
				// Interpolation Strategy
				String interMethod = "";
				String interConst = "";
				InterpolationStrategy is = curMap.getInterpolationType();
				if (is instanceof LinearIStrategy) {
					interMethod = XMLConstants.LINEAR;
				} else if (is instanceof MetricExponentialIStrategy) {
					interMethod = XMLConstants.METEXP;
					interConst = ((MetricExponentialIStrategy) is)
							.getConstant();
				} else if (is instanceof AugmentExponentialIStrategy) {
					interMethod = XMLConstants.AUGEXP;
					interConst = ((AugmentExponentialIStrategy) is)
							.getConstant();
				} else if (is instanceof MetricLogIStrategy) {
					interMethod = XMLConstants.METLOG;
					interConst = ((MetricLogIStrategy) is).getConstant();
				} else if (is instanceof AugmentLogIStrategy) {
					interMethod = XMLConstants.AUGLOG;
					interConst = ((AugmentLogIStrategy) is).getConstant();
				}
				if (interMethod.length() > 0)
					curMapEl
							.setAttribute(XMLConstants.INTERMETHOD, interMethod);
				if (interConst.length() > 0)
					curMapEl.setAttribute(XMLConstants.INTERCONST, interConst);

				emappings.add(curMapEl);
			}
		}
		return emappings;
	}
}
