package metricsOverlay.xml;

import metricsOverlay.model.MetricDocument;

import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.Path;

/**
 * This singleton class manages the stored location of XML files for the
 * plug-in.
 * 
 * @author Matthew Harward
 * @version 26/6/2009
 */
public class XMLManager {

	// Singleton Instance
	private static XMLManager manager;

	// Private Constructor - Singleton
	private XMLManager() {
	}

	/**
	 * Returns an instance of this singleton class.
	 * 
	 * @return an instance of the class.
	 */
	public static XMLManager getInstance() {
		if (manager == null)
			manager = new XMLManager();
		return manager;
	}

	/**
	 * This method locates the XML for a Java file used in eclipse, reads in the
	 * XML and creates a document representing the metrics to be overlaid.
	 * 
	 * @param path
	 *            the absolute path to the Java file.
	 * @return a document containing all of the metrics information.
	 */
	public MetricDocument locateXML(IPath path) {
		MetricXMLReader r = new MetricXMLReader(path.toOSString());
		r.readXML(extractXMLPath(path));
		return r.getMetricDocument();
	}

	/**
	 * This method extracts the appropriate XML path, given the absolute path of
	 * a java file. If the Eclipse project's source code is in a file named
	 * "src" the code is placed in an identical adjacent directory structure,
	 * "metricsxml". If this is not the case, for each directory (effectively
	 * per package) containing Java files, a sub-directory is created called
	 * "packagemetricsxml". These directory's contain all of the XML information
	 * that can be used by the program. Any applications seeking to produce XML
	 * for this plug-in need to populate these directories.
	 * 
	 * @param path
	 *            the absolute path of the Java file.
	 */
	public static String extractXMLPath(IPath path) {
		// Generate XML Path
		IPath xml = path.removeFileExtension().addFileExtension(XMLConstants.XML);
		int segPos = -1;
		for (int i = 0; i < xml.segmentCount(); i++) {
			// Gets last instance of src section
			if (xml.segment(i).equals(XMLConstants.SOURCEFOLDERNAME))
				segPos = i;
		}
		IPath xmlPath = null;
		// If there is a src section
		if (segPos >= 0) {
			xmlPath = xml.removeLastSegments(xml.segmentCount() - segPos);
			xmlPath = xmlPath.append(Path.fromOSString(XMLConstants.STDXMLDIR));
			xmlPath = xmlPath.append(xml.removeFirstSegments(segPos + 1));
		// If there isn't a src section
		} else {
			xmlPath = xml.removeLastSegments(1);
			xmlPath = xmlPath.append(Path.fromOSString(XMLConstants.NOSRCXMLDIR));
			xmlPath = xmlPath.append(xml
					.removeFirstSegments(xml.segmentCount() - 1));
		}
		String finalStr = xmlPath.toOSString();
		// Paths often start with file, this is impossible for certain applications
		// to read, so is removed.
		if (finalStr.startsWith("file:"))
			finalStr = finalStr.substring(6);
		//System.out.println(finalStr);
		return finalStr;
	}
	
	public static String getAbsoluteDTDPath() {
		return getAbsoluteLocalPath(XMLConstants.DTD);
	}
	
	public static String getAbsoluteLocalPath(String localRelativePath) {
		IPath aPath = Path.fromPortableString(System.getProperty("user.dir") + 
				localRelativePath);
		return aPath.toOSString();
	}

}
